﻿fn sqr x = x*x

fn checkProductRenderer = (
	productRenderer = renderers.production
	local enabled = false
	if(productRenderer != undefined) then (
		enabled = ((productRenderer.classid[1] == 1941615238) and (productRenderer.classid[2] == 2012806412) or (productRenderer.classid[1] == 1770671000) and (productRenderer.classid[2] == 1323107829))
	) 
	enabled
)
	
fn getMaxReleaseAsString=(
	local ver=maxversion()
	local result
	
	if (ver[1]>=10000) then (
		result=((((ceil (ver[1]/1000.0))-2+2000) as Integer) as String)
	)
	else (
		result="R"+(((ver[1]/1000) as Integer) as String)
	)
	result
)

fn getVRayInstallPath release=(
		local regKeyName="SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall"
		local regKey
		local subKey
		local result
		
		try (
			if (registry.openKey HKEY_LOCAL_MACHINE regKeyName accessRights:#readOnly key:&regKey) do (
				local subKeyNames=#()
				registry.getSubKeyNames regKey names: &subKeyNames
				
				local i=1
				for i=1 to (subKeyNames.count) do (
					if (matchPattern subKeyNames[i] pattern:("V-Ray for 3dsmax "+release+" for*")) do (
						registry.openKey regKey subKeyNames[i] accessRights:#readOnly key:&subKey
						registry.queryValue subKey "UninstallString" value:&result
						
						registry.closeKey subKey
					)
				)
				
				registry.closeKey regKey
			)
		)
		catch (
			if (subKey!=undefined) do (registry.closeKey subKey)
			if (regKey!=undefined) do (registry.closeKey regKey)
		)
		
		if (result!=undefined) do (
			result=substring result 2 ((findString result "\\uninstall\\installer.exe")-2)
		)
		
		result
)

-- used to look up materials that have already been converted
orig_mtls=#()
new_mtls=#()

-- used to look up textures that have already been converted
orig_texmaps=#()
new_texmaps=#()

-- used to look up bitmap textures that have already been converted
orig_bitmaptextures=#()
new_hdritexmaps=#()
			
maxver = (maxVersion())[1]
	
convertToVRay -- Placeholder for the function, otherwise maxscript returns error when executing
					-- the rollout's convert button event if evaluated with the function defined
					-- afterwards.

rollout vraySceneConverterRollout "V-Ray scene converter" width: 460 height: 180 (
	label label_message "All scene materials and raytraced shadows will be converted to V-Ray materials and shadows." width:400 height:26 align:#center offset:[0, 10]
	
	checkbox check_optReflections "Turn on \"trace reflections\" when converting Standard materials" checked:false width:400 align:#center offset:[0, 10]
	checkbox check_optLights "Convert mr Sun/Sky to V-Ray Sun/Sky" checked:false width:400 align:#center offset:[0, 0]
	checkbox check_optCameras "Convert standard cameras to physical cameras" checked:false width:400 align:#center
	checkbox check_optStdMtlOpacityToVRayMtlRefract "Convert Standard material opacity to V-Ray material refraction" checked:false width:400 align:#center
	checkbox check_optConvertOnlySelectedNodes "Convert selected nodes only" checked:false width:400 align:#center
	
	button btn_convertStart "Convert" width:60 align:#right across:2 offset:[150, 0]
	button btn_convertCancel "Cancel" width:60 align:#right
	
	on btn_convertStart pressed do (
		convertToVRay  check_optConvertOnlySelectedNodes.checked
		destroyDialog vraySceneConverterRollout
	)
	
	on btn_convertCancel pressed do (
		destroyDialog vraySceneConverterRollout
	)
)

fn convertFrom_Standard origMtl = (
	r=VRayMtl()
	r.name="vray "+origMtl.name
	
	r.texmap_bump=origMtl.bumpMap
	if (r.texmap_bump!=undefined) then (
		r.texmap_bump_on=origMtl.bumpMapEnable
		r.texmap_bump_multiplier=origMtl.bumpMapAmount
	)

	r.texmap_displacement=origMtl.displacementMap
	if (r.texmap_displacement!=undefined) then (
		r.texmap_displacement_on=origMtl.displacementMapEnable
		r.texmap_displacement_multiplier=origMtl.displacementMapAmount
	)

	r.diffuse=origMtl.diffuse
	r.texmap_diffuse=origMtl.diffuseMap
	if (r.texmap_diffuse!=undefined) then (
		r.texmap_diffuse_on=origMtl.diffuseMapEnable
		r.texmap_diffuse_multiplier=origMtl.diffuseMapAmount
	)

	if (origMtl.shaderByName !="Strauss") then (
		r.reflection=origMtl.specular*origMtl.specularLevel*0.01
	)
	
	if (vraySceneConverterRollout.check_optStdMtlOpacityToVRayMtlRefract.checked) do (
		r.refraction=(Color 255 255 255) * (1.0 - origMtl.opacity*0.01)
	)
	
	K=2.0^(origMtl.glossiness*0.01*10.0)
	if (origMtl.shaderType==5) then (
		-- phong
		r.brdf_type=0
		r.reflection=r.reflection*2.0/(K+1.0)
	) else (
		-- blinn
		r.brdf_type=1
		K=K*4
		r.reflection=r.reflection*8.0/(K+1.0)
	)
	
	r.reflection_lockGlossiness=false
	r.hilight_glossiness=1.0-(1.0/(K+1.0))^(1.0/3.5)
	r.reflection_glossiness=1.0
	r.option_traceReflection=vraySceneConverterRollout.check_optReflections.checked
	
	r.texmap_opacity=origMtl.opacityMap
	if (r.texmap_opacity!=undefined) then (
		r.texmap_opacity_on=origMtl.opacityMapEnable
		r.texmap_opacity_multiplier=origMtl.opacityMapAmount
	)
	
	r
)

fn convertFrom_Raytrace origMtl = (
	r=VRayMtl()
	r.name="vray "+origMtl.name

	r.texmap_bump=origMtl.bumpMap
	if (r.texmap_bump!=undefined) then (
		r.texmap_bump_on=origMtl.bumpMapEnable
		r.texmap_bump_multiplier=origMtl.bumpMapAmount
	)

	r.texmap_displacement=origMtl.displacementMap
	if (r.texmap_displacement!=undefined) then (
		r.texmap_displacement_on=origMtl.displacementMapEnable
		r.texmap_displacement_multiplier=origMtl.displacementMapAmount
	)

	r.diffuse=origMtl.diffuse
	r.texmap_diffuse=origMtl.diffuseMap
	if (r.texmap_diffuse!=undefined) then (
		r.texmap_diffuse_on=origMtl.diffuseMapEnable
		r.texmap_diffuse_multiplier=origMtl.diffuseMapAmount
	)

	r.reflection=origMtl.spec__color*origMtl.specular_level*0.01*0.5
	
	K=2.0^(origMtl.glossiness*0.01*10.0)
	if (origMtl.shaderType==1) then (
		-- phong
		r.brdf_type=0
		r.reflection=r.reflection*2.0/(K+1.0)
	) else (
		-- blinn
		r.brdf_type=1
		K=K*4
		r.reflection=r.reflection*8.0/(K+1.0)
	)

	if (origMtl.reflect.r>r.reflection.r) then r.reflection.r=origMtl.reflect.r
	if (origMtl.reflect.g>r.reflection.g) then r.reflection.g=origMtl.reflect.g
	if (origMtl.reflect.b>r.reflection.b) then r.reflection.b=origMtl.reflect.b
	
	r.reflection_lockGlossiness=false
	r.hilight_glossiness=1.0-(1.0/(K+1.0))^(1.0/3.5)
	
	if ((origMtl.reflect.r+origMtl.reflect.g+origMtl.reflect.b)/3.0<1e-6) then r.option_traceReflection=false
	
	r.texmap_reflection=origMtl.reflectionMap
	if (r.texmap_reflection!=undefined) then (
		r.texmap_reflection_on=origMtl.reflectionMapEnable
		r.texmap_reflection_multiplier=origMtl.reflectionMapAmount
	)
	
	r.refraction=origMtl.transparecy
	r.refraction_ior=origMtl.Index_of_Refraction

	r.texmap_refraction=origMtl.transparencyMap
	if (r.texmap_refraction!=undefined) then (
		r.texmap_refraction_on=origMtl.transparencyMapEnable
		r.texmap_refraction_multiplier=origMtl.transparencyMapAmount
	)

	r
)

fn convertFrom_fR_Advanced origMtl = (
	r=VRayMtl()
	r.name="vray "+origMtl.name
	
	-- diffuse
	r.diffuse=origMtl.diffuseColor
	r.texmap_diffuse=origMtl.diffuseMap
	r.texmap_diffuse_on=origMtl.useDiffuseMap
	r.texmap_diffuse_multiplier=origMtl.diffuseMapAmount
	
	-- bump map
	r.texmap_bump=origMtl.bumpMap
	r.texmap_bump_on=origMtl.useBumpMap
	r.texmap_bump_multiplier=origMtl.bumpMapAmount
	
	-- displacement
	r.texmap_displacement=origMtl.MTDMap
	r.texmap_displacement_on=origMtl.useMTDMap
	r.texmap_displacement_multiplier=origMtl.MTDMapAmount

	-- reflection
	r.reflection=origMtl.reflectFilter
	r.texmap_reflection=origMtl.reflectMap
	r.texmap_reflection_on=origMtl.useReflectMap
	r.texmap_reflection_multiplier=origMtl.reflectMapAmount
	
	r.reflection_glossiness=1.0-sqr (origMtl.reflectGlossy*0.01)
	r.texmap_reflectionGlossiness=origMtl.reflectBlurryMap
	r.texmap_reflectionGlossiness_on=origMtl.useReflectBlurryMap
	r.texmap_reflectionGlossiness_multiplier=origMtl.reflectBlurryMapAmount
	
	r.reflection_subdivs=sqrt origMtl.reflectSamples
	r.reflection_fresnel=origMtl.useFresnel
	r.reflection_useInterpolation=origMtl.useReflBlur
	
	-- refraction
	r.refraction=origMtl.refractFilter
	r.texmap_refraction=origMtl.refractMap
	r.texmap_refraction_on=origMtl.useRefractMap
	r.texmap_refraction_multiplier=origMtl.refractMapAmount
	
	r.refraction_ior=origMtl.refractIOR
	r.texmap_refractionIOR=origMtl.refractIORMap
	r.texmap_refractionIOR_on=origMtl.useRefractIORMap
	r.texmap_refractionIOR_multiplier=origMtl.refractIORMapAmount
	
	r.refraction_glossiness=1.0-sqr (origMtl.refractGlossy*0.01)
	r.texmap_refractionGlossiness=origMtl.refractBlurryMap
	r.texmap_refractionGlossiness_on=origMtl.useRefractBlurryMap
	r.texmap_refractionGlossiness_multiplier=origMtl.refractBlurryMapAmount
	
	r.refraction_subdivs=sqrt origMtl.refractSamples
	r.refraction_useInterpolation=origMtl.useRefrBlur
	
	r
)

fn convertFrom_fR_Metal origMtl = (
	r=VRayMtl()
	r.name="vray "+origMtl.name
	
	r.diffuse=origMtl.diffuseColor
	r.texmap_diffuse=origMtl.diffuseMap
	r.texmap_diffuse_on=origMtl.diffuseMapEnable
	r.texmap_diffuse_multiplier=origMtl.diffuseMapAmount
	
	r.reflection=origMtl.reflectColor
	r.texmap_reflection=origMtl.reflectLevelMap
	r.texmap_reflection_on=origMtl.reflectLevelMapEnable
	r.texmap_reflection_multiplier=origMtl.reflectLevelMapAmount
	
	r
)

fn convertFrom_fR_Glass origMtl = (
	r=VRayMtl()
	r.name="vray "+origMtl.name
	
	r.diffuse=origMtl.diffuseColor
	r.texmap_diffuse=origMtl.diffuseMap
	r.texmap_diffuse_on=origMtl.diffuseMapEnable
	r.texmap_diffuse_multiplier=origMtl.diffuseMapAmount

	r.reflection=origMtl.reflectColor
	r.reflection_fresnel=origMtl.useFresnel
	r.texmap_reflection=origMtl.reflectMap
	r.texmap_reflection_on=origMtl.reflectMapEnable
	r.texmap_reflection_multiplier=origMtl.reflectMapAmount
	
	r.refraction=origMtl.refractColor
	r.refraction_ior=origMtl.refractIOR
	r.texmap_refraction=origMtl.refractMap
	r.texmap_refraction_on=origMtl.refractMapEnable
	r.texmap_refraction_multiplier=origMtl.refractMapAmount
	
	r.refraction_affectShadows=true
	r.refraction_affectAlpha=1
	
	r
)

fn convertFromBlend orig_mtl = (
	r=VRayBlendMtl()
	r.name="vray "+orig_mtl.name
	
	r.baseMtl=orig_mtl.map1
	r.coatMtl[1]=orig_mtl.map2
	r.blend[1]=(orig_mtl.mixAmount*0.01)*(color 255 255 255)
	if orig_mtl.maskEnabled then r.texmap_blend[1]=orig_mtl.mask
	
	r
)

fn convertFromShellac orig_mtl = (
	r=VRayBlendMtl()
	r.name="vray "+orig_mtl.name
	r.additiveMode=true
	
	r.baseMtl=orig_mtl.shellacMtl1
	r.coatMtl[1]=orig_mtl.shellacMtl2
	r.blend[1]=(orig_mtl.shellacColorBlend*0.01)*(color 255 255 255)
	
	r
)

fn convertFromDoubleSided orig_mtl = (
	r=VRay2SidedMtl()
	r.name="vray "+orig_mtl.name

	r.frontMtl=orig_mtl.material1
	r.backMtl=orig_mtl.material2
	r.translucency=orig_mtl.translucency*0.01*(color 255 255 255)
	r.backMtlOn=orig_mtl.map2Enabled
	
	r
)

fn convertFrom_Architectural orig_mtl = (
	r=VRayMtl()
	r.name="vray "+orig_mtl.name

	-- diffuse component
	r.diffuse=orig_mtl.diffuse
	r.texmap_diffuse=orig_mtl.diffuseMap
	if (r.texmap_diffuse!=undefined) then (
		r.texmap_diffuse_on=orig_mtl.diffuseMapEnable
		r.texmap_diffuse_multiplier=orig_mtl.diffuseMapAmount
	)

	-- specular component
	r.reflection=(color 255 255 255)
	r.texmap_reflection=orig_mtl.shininessMap
	r.reflection_fresnel=true
	r.hilight_glossiness=(orig_mtl.shininess*0.01)*0.25+0.75
	r.reflection_glossiness=1.0
	r.reflection_lockGlossiness=false

	-- refraction
	r.refraction=orig_mtl.transparency*0.01*(color 255 255 255)
	r.refraction_ior=orig_mtl.ior
	
	-- if there is luminance, wrap the material into a VRayBlendMtl and add a VRayLightMtl on top of it
	if orig_mtl.luminance>0 then (
		local l=VRayLightMtl()
		l.color=orig_mtl.diffuse
		l.multiplier=orig_mtl.luminance
		l.opacity_texmap=VRayColor()
		l.opacity_texmap.color=(color 0 0 0)
		
		local m=VRayBlendMtl()
		m.baseMtl=r
		m.coatMtl[1]=l
		m.blend[1]=(color 255 255 255)
		
		r=m
	)
	
	-- if there is translucency, wrap the material into a VRay2SidedMtl
	if orig_mtl.translucency>0 then (
		local m=VRay2SidedMtl()
		m.frontMtl=r
		m.translucency=orig_mtl.translucency*0.01*(color 255 255 255)
		r=m
	)
	
	r
)

fn convertFrom_ArchAndDesign orig_mtl = (
	r=VRayMtl()
	r.name="vray "+orig_mtl.name
	
	-- diffuse component
	r.diffuse=orig_mtl.diff_color*orig_mtl.diff_weight
	r.texmap_diffuse=orig_mtl.diff_color_map
	if (r.texmap_diffuse!=undefined) then (
		r.diffuse=(color 0 0 0)
		r.texmap_diffuse_on=orig_mtl.diff_color_map_on
		r.texmap_diffuse_multiplier=orig_mtl.diff_weight*100
	)
	
	-- reflection
	r.reflection=orig_mtl.refl_color*orig_mtl.refl_weight
	r.texmap_reflection=orig_mtl.refl_color_map
	if (r.texmap_reflection!=undefined) then (
		r.reflection=(color 0 0 0)
		r.texmap_reflection_on=orig_mtl.refl_color_map_on
		r.texmap_reflection_multiplier=orig_mtl.refl_weight*100
	)
	
	r.reflection_glossiness=orig_mtl.refl_gloss
	r.texmap_reflectionGlossiness=orig_mtl.refl_gloss_map
	if (r.texmap_reflectionGlossiness!=undefined) then (
		r.reflection_glossiness=0.0
		r.texmap_reflectionGlossiness_on=orig_mtl.refl_gloss_map_on
	)
	
	-- Fresnel options
	if (orig_mtl.refl_func_fresnel==true) then ( -- use Fresnel
		r.reflection_fresnel=true
	) else ( -- use falloff curve
		local tcolor=Falloff()
		if (r.texmap_reflection==undefined) then (
			tcolor.color1=orig_mtl.refl_color*orig_mtl.refl_func_low
			tcolor.color2=orig_mtl.refl_color*orig_mtl.refl_func_high
		) else (
			tcolor.color1=(color 1.0 1.0 1.0)*orig_mtl.refl_func_low
			tcolor.color2=(color 1.0 1.0 1.0)*orig_mtl.refl_func_high
			
			local m=VRayCompTex()
			m.sourceA=r.texmap_reflection
			m.sourceB=tcolor
			m.operator=3
			tcolor=m
		)
		
		r.texmap_reflection=tcolor
		r.texmap_reflection_on=true
		r.texmap_reflection_multiplier=orig_mtl.refl_weight*100
		r.reflection=(color 0 0 0)
	)
	
	-- refraction/opacity
	r.refraction_ior=orig_mtl.refr_ior
	r.refraction_affectShadows=not orig_mtl.opts_do_refractive_caustics
	r.refraction_affectAlpha=(if orig_mtl.opts_propagate_alpha then 1 else 0)
	if (orig_mtl.opts_1sided==on) then ( -- opacity
		if (orig_mtl.refr_color_map!=undefined) then ( -- there is a transparency texture
			local tcolor=Output()
			tcolor.map1=orig_mtl.refr_color_map
			tcolor.output.invert=true
			tcolor.output_amount=orig_mtl.refr_weight
			
			r.texmap_opacity=tcolor
		) else ( -- no transparency texture, just color
			local c=orig_mtl.refr_color
			if (orig_mtl.refr_weight>0 and (c.r>0 or c.g>0 or c.b>0)) then ( -- there is some transparency
				local tcolor=VRayColor()
				tcolor.color=(color (1.0-orig_mtl.refr_color.r) (1.0-orig_mtl.refr_color.g) (1.0-orig_mtl.refr_color.b))
				r.texmap_opacity=tcolor
			)
		)
	) else ( -- refraction
		r.refraction=orig_mtl.refr_color*orig_mtl.refr_weight
		r.texmap_refraction=orig_mtl.refr_color_map
		if (r.texmap_refraction!=undefined) then (
			r.refraction=(color 0 0 0)
			r.texmap_refraction_on=orig_mtl.refr_color_map_on
			r.texmap_refraction_multiplier=orig_mtl.refr_weight*100
		)
	)
	
	r
)

fn convertFrom_CoronaMtl orig_mtl = (
	r=VRayMtl()
	r.name="vray "+orig_mtl.name
	
	--Diffuse
	r.diffuse = orig_mtl.levelDiffuse * orig_mtl.colorDiffuse
	r.texmap_diffuse=orig_mtl.texmapDiffuse
	if r.texmap_diffuse != undefined then(
		r.texmap_diffuse_on = orig_mtl.texmapOnDiffuse
		r.texmap_diffuse_multiplier = orig_mtl.mapAmountDiffuse
	)
	
	--Reflection
	r.reflection = orig_mtl.levelReflect  * orig_mtl.colorReflect
	r.texmap_reflection = orig_mtl.texmapReflect
	if r.texmap_reflection != undefined then(
		r.texmap_reflection_on = orig_mtl.texmapOnReflect
		r.texmap_reflection_multiplier = orig_mtl.mapamountReflect
	)
	
	r.reflection_fresnel = true
	if orig_mtl.fresnelIor == orig_mtl.Ior then
	(
		r.reflection_lockIOR = true
		r.reflection_ior = orig_mtl.ior
		r.refraction_ior = orig_mtl.ior
	)else(
		r.reflection_lockIOR = false
		r.reflection_ior = orig_mtl.fresnelIor 
	) 
					
	r.texmap_reflectionIOR = orig_mtl.texmapFresnelIor
	if r.texmap_reflectionIOR != undefined then(
		r.texmap_reflectionIOR_on = orig_mtl.texmapOnFresnelIor
		r.texmap_reflectionIOR_multiplier = orig_mtl.mapamountFresnelIor
	)
	
	r.reflection_glossiness = orig_mtl.reflectGlossiness
	r.texmap_reflectionGlossiness = orig_mtl.texmapReflectGlossiness
	if r.texmap_reflectionGlossiness != undefined then(
		r.texmap_reflectionGlossiness_on = orig_mtl.texmapOnReflectGlossiness
		r.texmap_reflectionGlossiness_multiplier = orig_mtl.mapamountReflectGlossiness
	)
	
	--Anisotropy
	r.anisotropy = orig_mtl.anisotropy  
	r.texmap_anisotropy = orig_mtl.texmapReflectAnisotropy
	if r.texmap_anisotropy != undefined then(
		r.texmap_anisotropy_on = orig_mtl.texmapOnAnisotropy
		r.texmap_anisotropy_multiplier = orig_mtl.mapamountAnisotropy
	)
	r.anisotropy_rotation = orig_mtl.anisotropyRotation  
	r.texmap_anisotropy_rotation = orig_mtl.texmapReflectAnisotropyRotation
	if r.texmap_anisotropy_rotation != undefined then(
		r.texmap_anisotropy_rotation_on = orig_mtl.texmapOnAnisotropyRotation
		r.texmap_anisotropy_rotation_multiplier = orig_mtl.mapamountAnisotropyRotation
	)
	
	--Refraction
	r.refraction = orig_mtl.levelRefract*orig_mtl.colorRefract
	r.texmap_refraction = orig_mtl.texmapRefract
	if r.texmap_refraction != undefined then(
		r.texmap_refraction_on = orig_mtl.texmapOnRefract
		r.texmap_refraction_multiplier = orig_mtl.mapamountRefract
	)
	
	r.refraction_ior = orig_mtl.ior
	r.texmap_refractionIOR = orig_mtl.texmapIor
	if r.texmap_refractionIOR != undefined then(
		r.texmap_refractionIOR_on = orig_mtl.texmapOnIor
		r.texmap_refractionIOR_multiplier = orig_mtl.mapamountIor
	)
	
	r.refraction_glossiness = orig_mtl.refractGlossiness
	r.texmap_refractionGlossiness = orig_mtl.texmapRefractGlossiness
	if r.texmap_refractionGlossiness != undefined then(
		r.texmap_refractionGlossiness_on = orig_mtl.texmapOnRefractGlossiness
		r.texmap_refractionGlossiness_multiplier = orig_mtl.mapamountRefractGlossiness
	)
	
	r.refraction_affectShadows = not orig_mtl.caustics
	if  orig_mtl.thin do r.refraction_ior = 1.0
		
	--Opacity
	r.texmap_opacity = orig_mtl.texmapOpacity
	if r.texmap_opacity != undefined then(
		r.texmap_opacity_on = orig_mtl.texmapOnOpacity
		r.texmap_opacity_multiplier = orig_mtl.mapamountOpacity
	)
	
	--Bumpmap
	r.texmap_bump = orig_mtl.texmapBump
	if r.texmap_bump != undefined then(
		r.texmap_bump_on = orig_mtl.texmapOnBump
		r.texmap_bump_multiplier = 100.0*orig_mtl.mapamountBump
	)
		
	--Absorption
	r.refraction_fogMult  = if orig_mtl.absorptionDistance == 0.0 then 1.0 else 1.0/orig_mtl.absorptionDistance
	r.refraction_fogColor  = orig_mtl.absorptionColor
		
	--Displacement
	r.texmap_displacement = orig_mtl.texmapDisplace
	if r.texmap_displacement != undefined then(
		r.texmap_displacement_on = orig_mtl.texmapOnDisplacement
		r.texmap_displacement_multiplier = (orig_mtl.displacementMaximum - orig_mtl.displacementMinimum) --(Displace_amount) : worldUnits
	)		
		
	--Self illumination
	r.selfIllumination_multiplier = orig_mtl.levelSelfIllum
	r.selfIllumination = if orig_mtl.levelSelfIllum == 0.0 then black else orig_mtl.colorSelfIllum
	r.texmap_self_illumination = orig_mtl.texmapSelfIllum
	if r.texmap_self_illumination != undefined then(
		r.texmap_self_illumination_on = orig_mtl.texmapOnSelfIllum
		r.texmap_self_illumination_multiplier = orig_mtl.mapamountSelfIllum
	)
	
	-- Translucency
	if orig_mtl.levelTranslucency > 0.0 then(
		newVRay2SidedMtl = VRay2SidedMtl name:("vray "+orig_mtl.name)
		newVRay2SidedMtl.frontMtl = r
		
			if orig_mtl.texmapTranslucency == undefined then(
				newVRay2SidedMtl.texmap_translucency_multiplier = 100.0*orig_mtl.levelTranslucency
				newVRay2SidedMtl.Translucency = orig_mtl.colorTranslucency
			)
			if orig_mtl.texmapTranslucency != undefined and orig_mtl.texmapOnTranslucency == true then(
				theMix = Mix map1: orig_mtl.texmapTranslucency color2: orig_mtl.colorTranslucency mixAmount: orig_mtl.mapamountTranslucency
				newVRay2SidedMtl.texmap_translucency_multiplier = 100.0*orig_mtl.levelTranslucency
				newVRay2SidedMtl.Translucency = white
				newVRay2SidedMtl.texmap_translucency = theMix
			)
		return newVRay2SidedMtl
	) else return r	
)

fn convertFrom_CoronaLightMtl orig_mtl = (
	r=VrayLightMtl()
	r.name="vray "+orig_mtl.name
	
	r.color = orig_mtl.color 
	r.multiplier = orig_mtl.intensity
	r.directLight_on = orig_mtl.emitLight
	
	if orig_mtl.texmap != undefined then(
		r.texmap_on = orig_mtl.texmapOn 
		r.texmap = orig_mtl.texmap 
	)
	if orig_mtl.opacityTexmap != undefined then(
		r.opacity_texmap_on = orig_mtl.opacityTexmapOn 
		r.opacity_texmap = orig_mtl.opacityTexmap 
	)
	return r
) 

fn convertFrom_CoronaRaySwitchMtl orig_mtl = (
	r = VRayOverrideMtl()
	r.name="vray "+orig_mtl.name
	
	r.baseMtl = orig_mtl.directMtl
	r.giMtl = orig_mtl.giMtl
	r.reflectMtl = orig_mtl.reflectMtl
	r.refractMtl = orig_mtl.refractMtl 
	
	return r
)
	
fn convertFrom_CoronaVolumeMtl orig_mtl = (
	vr=VRayScatterVolume()
	vr.name="vray "+orig_mtl.name
	
	vr.overall_color = orig_mtl.absorptionColor 
	vr.sss_color = orig_mtl.scatteringColor
	vr.scatter_color = orig_mtl.absorptionColor 
	vr.phase_function = orig_mtl.scatteringDirectionality
	vr.scatter_radius = orig_mtl.absorptionDist
	if orig_mtl.singleBounce then vr.bounces = 1
	
	if orig_mtl.emissionDist > 0 then (
		vrLightMtl=VRayLightMtl()
		vrLightMtl.name = "vraylight "+orig_mtl.name
		vrLightMtl.color = orig_mtl.emissionColor
		vrLightMtl.multiplier = (1.0f / orig_mtl.emissionDist)
		
		vrBlendMtl = VRayBlendMtl()
		vrBlendMtl.name = "vrayblend "+orig_mtl.name
		vrBlendMtl.baseMtl = vr
		vrBlendMtl.coatMtl[1] = vrLightMtl
		return vrBlendMtl
	)
	
	return vr
) 

fn convertFrom_CoronaShadowCatcherMtl orig_mtl = (
	vrwrap=VRayMtlWrapper()
	vrwrap.name="vray "+orig_mtl.name
	
	vrwrap.matteSurface = true
	vrwrap.alphaContribution = case orig_mtl.alphaMode of (
		1: 1.0f
		2: -1.0f
		3: 0.0f
		default: -1.0f
	)
	vrwrap.matte_shadows = true
	vrwrap.matte_shadowsAffectAlpha = true
	vrwrap.matte_shadowBrightness = orig_mtl.shadowAmount
	
	vr=VRayMtl()
	vr.diffuse = orig_mtl.backplateColor
	if orig_mtl.backplateTexmap != undefined then (
		vr.texmap_diffuse = orig_mtl.backplateTexmap
		vr.texmap_diffuse_on = true
	)
	
	--Reflection
	vr.reflection = orig_mtl.reflectLevel  * orig_mtl.colorReflect
	vr.texmap_reflection = orig_mtl.reflectColorTexmap 

	vr.reflection_fresnel = true
	vr.reflection_lockIOR = false
	vr.reflection_ior = orig_mtl.reflectIor 
	vr.reflection_glossiness = orig_mtl.reflectGlossiness
	
	vr.texmap_bump = orig_mtl.bump
	if vr.texmap_bump != undefined then(
		vr.texmap_bump_on = true
		vr.texmap_bump_multiplier = 100.0*orig_mtl.bumpAmount
	)
	
	vrwrap.baseMtl = vr
	return vrwrap
)

fn convertFrom_CoronaColorTex inTexture = (
	vrayC = VRayColor name:inTexture.name alpha:inTexture.alpha rgb_multiplier: inTexture.multiplier
	if inTexture.method == 0 do (
		vrayC.color_mode = 0
		vrayC.color = inTexture.color
	)
	if inTexture.method == 1 do (
		vrayC.color_mode = 0
		vrayC.red = inTexture.colorHdr[1]
		vrayC.green = inTexture.colorHdr[2]
		vrayC.blue = inTexture.colorHdr[3]
	)
	if inTexture.method == 2 do (
		vrayC.color_mode = 1
		vrayC.temperature = inTexture.temperature
	)
	if inTexture.method == 3 do (
		vrayC.color_mode = 0
		local strHexColor = inTexture.hexColor
		if (strHexColor.count == 6 or strHexColor.count == 7) then (
			local startVal = 1
			if ((findString strHexColor "#") == 1) do startVal = 2
			
			local subStrHexColor = "0x" + (substring strHexColor startVal 2)
			local rColor = (subStrHexColor as float)
			startVal = startVal +2
			subStrHexColor = "0x" + (substring strHexColor startVal 2)
			local gColor = (subStrHexColor as float)
			startVal = startVal +2
			subStrHexColor = "0x" + (substring strHexColor startVal 2)
			local bColor = (subStrHexColor as float)
			
			if (rColor!=undefined and gColor!=undefined and bColor!=undefined) then (
				vrayC.color = (color rColor gColor bColor)
			) else (
				vrayC.color = (color 0.0f 0.0f 0.0f)
			)
		) else (
			vrayC.color = (color 255.0f 0.0f 0.0f)
		)
	)
	return  vrayC
)

fn convertFrom_CoronaMultiMapTex inTexture = (
	vrayMST = VRayMultiSubTex name:inTexture.name
	if vrayMST.texmap_num > inTexture.items do (
		local removeItems = vrayMST.texmap_num - inTexture.items
		vrayMST.deleteSubtex (inTexture.items + 1) removeItems
	)
	
	if vrayMST.texmap_num < inTexture.items do (
		local addItems = inTexture.items - vrayMST.texmap_num
		vrayMST.addSubtex addItems
	)
	
	if inTexture.mode == 5 do vrayMST.from_id = 1
	if inTexture.mode == 2 do vrayMST.from_id = 6
	
	for i = 1 to inTexture.items do (
		vrayMST.texmap_color[i] = inTexture.colors[i]
		vrayMST.texmap[i] = inTexture.texmaps[i]
	)
	
	return vrayMST
)

fn convertFrom_CoronaMixTex inTexture = (
	vrayCT = VRayCompTex sourceA: inTexture.texmapTop sourceB: inTexture.texmapBottom name: inTexture.name
	vrayCT.operator = case  of (
											(inTexture.mixOperation == 0):0 
											(inTexture.mixOperation == 1):1 
											(inTexture.mixOperation == 2):3 
											(inTexture.mixOperation == 3):4
											(inTexture.mixOperation == 4):5 
											(inTexture.mixOperation == 5):6 
											default:0
											) 					
	return vrayCT
)

fn convertFrom_CoronaAoTex inTexture = (
	vrayDt = VrayDirt occluded_color:inTexture.colorOccluded unoccluded_color:inTexture.colorUnoccluded \
	radius:inTexture.maxDistance texmap_radius:inTexture.texmapDistance texmap_occluded_color:inTexture.texmapOccluded \ 
	texmap_unoccluded_color:inTexture.texmapUnoccluded name:inTexture.name 
	
	if (hasProperty inTexture "invert") then (
		vrayDt.invert_normal = inTexture.invert
	)
	else if (hasProperty inTexture "normalMode") then (
		vrayDt.invert_normal = case  of (
														(inTexture.normalMode == 0):false
														(inTexture.normalMode == 1):true
														(inTexture.normalMode == 2):false
														default:false
													) 					
	)
	
	if inTexture.excludeMode == 1 and inTexture.excludeList.count != 0 do vrayDt.excludeList = inTexture.excludeList
	if inTexture.excludeMode == 3 do vrayDt.consider_same_object_only = true 
	
	vrayDt.texmap_radius = if inTexture.texmapDistance != undefined do inTexture.texmapDistance
	vrayDt.texmap_occluded_color = if inTexture.texmapOccluded != undefined do inTexture.texmapOccluded
	vrayDt.texmap_unoccluded_color = if inTexture.texmapUnoccluded != undefined do inTexture.texmapUnoccluded
	
	return vrayDt
)

fn convertFrom_CoronaNormalTex inTexture = (
	vrayNM = VrayNormalMap normal_map:inTexture.normalMap 
	try( vrayNM.map_channel =  inTexture.normalMap.coords.mapChannel
		)catch()
			
	return vrayNM
)

fn isVRayCompatible mtl_classid = (
	-- native V-Ray materials
	if mtl_classid==VRayMtl then return true
	if mtl_classid==VRay2SidedMtl then return true
	if mtl_classid==VRayBlendMtl then return true
	if mtl_classid==VRayFastSSS then return true
	if mtl_classid==VRayLightMtl then return true
	if mtl_classid==VRayMtlWrapper then return true
	
	-- supported 3dsmax materials
	if mtl_classid==Multimaterial then return true
	if mtl_classid==Arch___Design__mi then return true
	
	false
)

fn isVRayCompatibleTex tex_classid = (
	-- native V-Ray textures
	compatibleTexClassIDs = #(VRayBmpFilter, VRayBump2Normal, VRayColor, VRayColor2Bump, VRayCompTex, VRayCurvature, VRayDirt, VRayDistanceTex, VRayEdgesTex, \
										  VRayFakeFresnelTex, VRayGLSLTex, VRayHairInfoTex, VRayHDRI, VRayICC, VRayLut, VRayMap, VRayMultiSubTex, VRayOCIO, VRayOSLTex, \
										  VRayParticleTex, VRayPtex, VRaySamplerInfoTex, VRaySky, VRaySoftbox, VRayTriplanarTex, VRayUserColor, VRayUserScalar)
    result = findItem compatibleTexClassIDs tex_classid
	if result > 0 then return true
	return false
)

fn createVRayTexture inTexture= (
	-- check if this is a texture; if not, just return the value
	if ((superclassof inTexture)!=textureMap) then return inTexture

	-- check if we have already converted this texture; if yes, just return
	-- the already converted one	
	local i=findItem orig_texmaps inTexture
	if i>0 then return new_texmaps[i]

	-- recurse into sub-textures, if any
	local sub_count=getNumSubTexmaps inTexture
	if sub_count>0 then (
		local sub_tex = undefined
		for i=1 to sub_count do (
			sub_tex = getSubTexmap inTexture i
			if(sub_tex != undefined) then
				setSubTexmap inTexture i (createVRayTexture sub_tex)
		)
	)

	-- get the texture class id
	cls=classof inTexture
	
	-- if this is already a V-Ray compatible material, return the same one
	if (isVRayCompatibleTex cls) then return inTexture
	
	-- try to convert
	format "  %\n" inTexture
	local new_tex=case cls of (
		Normal_Bump: VRayNormalMap name:inTexture.name normal_map:inTexture.normal_map normal_map_on:inTexture.map1on normal_map_multiplier:inTexture.mult_spin \
											bump_map:inTexture.bump_map bump_map_on:inTexture.map2on bump_map_multiplier:inTexture.bump_spin \
											flip_red:inTexture.flipred flip_green:inTexture.flipgreen swap_red_and_green:inTexture.swap_rg
		
		CoronaOutput: if inTexture.inputTexmap != undefined do inTexture.inputTexmap
		CoronaRaySwitch: VRayColor color:red name:inTexture.name
		CoronaFrontBack: VRayColor color:inTexture.frontColor name:inTexture.name
		CoronaColor: convertFrom_CoronaColorTex inTexture
		CoronaMultiMap: convertFrom_CoronaMultiMapTex inTexture
		CoronaMix: convertFrom_CoronaMixTex inTexture
		CoronaAo: convertFrom_CoronaAoTex inTexture
		CoronaNormal: convertFrom_CoronaNormalTex inTexture
		coronasky: VraySky name:inTexture.name
		CoronaWire: VRayEdgesTex edgesColor:inTexture.edgeColor widthType:(if inTexture.widthMode == 1 then 0 else 1) thickness:inTexture.edgeWidthWorld \
											pixelWidth:inTexture.edgeWidthPixels name:inTexture.name
		
		default: inTexture
	)
	
	-- if converted, add to the list of converted textures
	if new_tex!=inTexture then (
		append orig_texmaps inTexture
		append new_texmaps new_tex
	)
	
	-- return the new texture
	new_tex
)

fn createVRayMtl orig_mtl = (
	-- check if this is a material; if not, just return the value
	if ((superclassof orig_mtl)!=material) then return orig_mtl

	-- convert sub-textures of this material
	local sub_texs_count=getNumSubTexmaps orig_mtl
	if sub_texs_count>0 then (
		local sub_tex = undefined
		for i=1 to sub_texs_count do (
			sub_tex = getSubTexmap orig_mtl i
			if(sub_tex != undefined) then
				setSubTexmap orig_mtl i (createVRayTexture sub_tex)
		)
	)
	
	-- check if we have already converted this material; if yes, just return
	-- the already converted one	
	local i=findItem orig_mtls orig_mtl
	if i>0 then return new_mtls[i]

	-- recurse into sub-materials, if any
	local sub_count=getNumSubMtls orig_mtl
	if sub_count>0 then (
		local sub_mtl = undefined
		for i=1 to sub_count do (
			sub_mtl=getSubMtl orig_mtl i
			if(sub_mtl != undefined) then
				setSubMtl orig_mtl i (createVRayMtl sub_mtl)
		)
	)

	-- get the material class id
	cls=classof orig_mtl
	
	-- if this is already a V-Ray compatible material, return the same one
	if (isVRayCompatible cls) then return orig_mtl

	-- otherwise, try to convert
	format "  %\n" orig_mtl
	local new_mtl=case cls of (
		StandardMaterial: convertFrom_Standard orig_mtl
		RaytraceMaterial: convertFrom_Raytrace orig_mtl
		Architectural: convertFrom_Architectural orig_mtl
		
		fR_Advanced: convertFrom_fR_Advanced orig_mtl
		fR_Metal: convertFrom_fR_Metal orig_mtl
		fR_Glass: convertFrom_fR_Glass orig_mtl
		
		Blend: convertFromBlend orig_mtl
		Shellac: convertFromShellac orig_mtl
		DoubleSided: convertFromDoubleSided orig_mtl
		
		CoronaMtl: convertFrom_CoronaMtl orig_mtl
		CoronaLightMtl: convertFrom_CoronaLightMtl orig_mtl
		CoronaRaySwitchMtl: convertFrom_CoronaRaySwitchMtl orig_mtl
		CoronaPortalMtl: VrayMtl name:("vray" + orig_mtl.name)
		CoronaVolumeMtl: convertFrom_CoronaVolumeMtl orig_mtl
		CoronaShadowCatcherMtl: convertFrom_CoronaShadowCatcherMtl orig_mtl
		
		
		-- Do not convert Arch&Design materials - V-Ray supports them directly
		-- Arch___Design__mi: convertFrom_ArchAndDesign orig_mtl
		
		default: orig_mtl
	)
	
	-- if converted, add to the list of converted materials
	if new_mtl!=orig_mtl then (
		append orig_mtls orig_mtl
		append new_mtls new_mtl
	)
	
	-- return the new material
	new_mtl
)

fn replaceMtlNode node = (
	if isValidNode node then (
		local m=createVRayMtl node.material
		node.material=m
		format "    %\n" node.name
	)
	for i=1 to node.children.count do (
		replaceMtlNode node.children[i]
	)
)

-- convert all scene materials to VRay materials
fn mtlsToVRay selectedOnly= (
	format "Converting scene materials...\n"
	orig_mtls=#()
	new_mtls=#()
	orig_texmaps=#()
	new_texmaps=#()
	if selectedOnly then (
		format "Converting scene materials for selected nodes...\n"
		local numNodes = selection.count
		for iNode=1 to numNodes do (
			print selection[iNode]
			replaceMtlNode selection[iNode]
		)
	) else (
		replaceMtlNode rootNode
		
		format "Converting Material Editor materials...\n"
		for i=1 to 24 do (
			local m=createVRayMtl meditmaterials[i]
			meditmaterials[i]=m
		)
	)
	orig_mtls=#()
	new_mtls=#()
	orig_texmaps=#()
	new_texmaps=#()
	format "Done\n"
)

fn deselectSMENodes = (
	if (maxver < 13000) do return undefined
		
	local numViews = sme.GetNumViews() 
	for iView=1 to numViews do (
		local currentView = sme.GetView iView
		local numNodes = currentView.GetNumNodes()
		for iNode=1 to numNodes do (
			local currentNode = currentView.GetNode iNode
			currentNode.selected = false
		)
	)
)

fn selectSMENodeByRef objRef = (
	if (maxver < 13000) do return undefined
	
	local numViews = sme.GetNumViews() 
	for iView=1 to numViews do (
		local currentView = sme.GetView iView
		local nodeByRef = currentView.GetNodeByRef objRef
		if (nodeByRef != undefined) then (
			nodeByRef.selected = true
			print nodeByRef.name
		)
	)
)

fn deleteSMESelectedNodesInEachView = (
	if (maxver < 13000) do return undefined
	
	local numViews = sme.GetNumViews() 
	for iView=1 to numViews do (
		local currentView = sme.GetView iView
		currentView.DeleteSelection()
	)
)

fn showMtlSlotInSME activeSlot= (
	
	local m = getMeditMaterial activeSlot
	local maxver = (maxVersion())[1]
	if (maxver >= 13000 and MatEditor.mode == #advanced) then (
		local vrmatViewIdx = sme.GetViewByName "VRayMtlView"
		if (vrmatViewIdx == 0) then (
			vrmatViewIdx = sme.CreateView "VRayMtlView"
		)
		local vrmatView = sme.GetView vrmatViewIdx
		
		if (vrmatView != undefined) then (
			vrmatView.CreateNode m [0,0]
		)
	)
)

fn warningToSelectGeometry theOperationName= (
	format "V-Ray Toolbar WARNING!\n"
	format "In order to create % you have to select geometry first!\n" theOperationName
)

fn shadowsNode node = (
	if (isValidNode node) then (
		scls=superclassof node
		if (scls==light) then (
			try (
				olds=node.shadowGenerator
				cls=classof olds
				if (cls!=shadowMap and cls!=VRayShadow) then (
					format "  %\n" node
					news=VRayShadow()
					node.shadowGenerator=news
				)
			)
			catch ()
		)
	)
	
	for i=1 to node.children.count do (
		shadowsNode node.children[i]
	)
)

-- convert all shadow types except shadow maps to VRay shadows
fn shadowsToVRay selectedOnly = (
	format "Converting shadows...\n"
	if selectedOnly then (
		format "Converting shadows for selected nodes...\n"
		local numNodes = selection.count
		for iNode=1 to numNodes do (
			print selection[iNode]
			shadowsNode selection[iNode]
		)
	) else (
		shadowsNode rootNode
	)
	format "Done\n"
)

fn replaceCoronaLightNodeVRayLight cLt = (
	print cLt.name
	local theVRL=CreateInstance VRayLight name:(cLt.name+"_VRayLight") on:cLt.on castShadows:cLt.occludeOtherLights invisible:(not cLt.visibleDirectly) \
	affect_reflections:(not cLt.visibleRefl) color_mode:cLt.colorMode  color:cLt.color targeted:cLt.targeted 
	
	if cLt.colorMode == 1 do theVRL.color_temperature = cLt.blackbodyTemp
	if cLt.colorMode == 2 do theVRL.texmap_on = true
	if cLt.texmap != undefined do theVRL.texmap = cLt.texmap
	
	if cLt.targeted == true do theVRL.target_distance = cLt.targetDistance
		
	theVRL.lightDistribution = cLt.directionality
			
	--Include/Exclude
	if (cLt.excludeList != undefined and cLt.excludeList.count !=0) do (
		theVRL.inclExclType = 1
		if cLt.excludeIncludeMod then theVRL.includeList = cLt.excludeList else theVRL.excludeList = cLt.excludeList
	)
	
	--Color/ Intensity
	theVRL.normalizeColor = case of
	(
		(cLt.intensityUnits == 0):0
		(cLt.intensityUnits == 1):1
		default:0
	)
	
	theVRL.multiplier = cLt.intensity
	
	--shape
	case cLt.shape  of 
	(
		1: ( --Plane
			theVRL.type = 0
			theVRL.size0 = cLt.width/2.0
			theVRL.size1 = cLt.height/2.0
		)
		0: ( --Sphere
			theVRL.type = 2
			theVRL.size0 = cLt.width
		)
		2: ( --Disc
			theVRL.type = 4
			theVRL.size0 = cLt.width/2.0
		)
		3: ( --Cylinder
			theVRL.type = 2
			theVRL.size0 = cLt.width
		)
	)
	
	cLt.name = (cLt.name+"_VRayLight")
	cLt.baseObject=theVRL
	theVRL
)

--here input param : 'ver' have to be presented as <number>.<number>.{<number>}
fn checkForPreviousCoronaVersion ver = (
	res = false
	if CoronaRenderer != undefined do (
		local currVersion = CoronaRenderer.getCoronaVersion()
		local comma_strVersion = filterstring currVersion ","
		local point_strVersion = filterstring comma_strVersion[1] "."
		local fVer = filterstring ver "."
		if point_strVersion.count > 1 and fVer.count > 1 then 
			res = ((point_strVersion[2] as integer) < (fVer[2] as integer))
	)
	return res
)

fn replaceCoronaSunLightNodeVRayLight coronaSLNode = (
	local sunObj = VRaySun name:(coronaSLNode.name+"_VRaySun") enabled:coronaSLNode.on intensity_multiplier:coronaSLNode.intensity size_multiplier:coronaSLNode.sizeMultiplier invisible:(not coronaSLNode.visibleDirectly)
	local targetObj = dummy name:(coronaSLNode.name+"_VRaySun.Target") pos:coronaSLNode.pos  --then target
	targetObj.boxsize = [2,2,2]
	targetObj.lookat = sunObj --this makes the target for the sunn out of dummy object
	
	if checkForPreviousCoronaVersion("1.2.1") then (
		sunObj.pos = coronaSLNode.pos + coronaSLNode.displaySize*[sin coronaSLNode.height, 0.0, cos coronaSLNode.height] -- xy
		sunObj.pos = [coronaSLNode.pos.x , coronaSLNode.pos.y, sunObj.pos.z]  + [cos coronaSLNode.sunRotation , sin coronaSLNode.sunRotation, 0.0]*sqrt(coronaSLNode.displaySize^2 - sunObj.pos.z^2) --z
		
		--animated sun
		if coronaSLNode.sunRotation.isAnimated == true or coronaSLNode.height.isAnimated == true do
		(
			animate on
			for t in animationRange.start to animationRange.end by 1 do
			(
				at time t
				(
					sunObj.pos = coronaSLNode.pos + coronaSLNode.displaySize*[sin coronaSLNode.height, 0.0, cos coronaSLNode.height]
					sunObj.pos = [coronaSLNode.pos.x , coronaSLNode.pos.y, sunObj.pos.z]  + [cos coronaSLNode.sunRotation , sin coronaSLNode.sunRotation, 0.0]*sqrt(coronaSLNode.displaySize^2 - sunObj.pos.z^2)
				)
			)
		)
	) else (
		sunObj.pos = coronaSLNode.pos
		targetObj.pos = coronaSLNode.target.pos
	)

	if (coronaSLNode.colorMode == 0) do (
		sunObj.color_mode = 1
		sunObj.filter_Color = coronaSLNode.colorDirect
	)
	
	--excludeList
	if coronaSLNode.excludeList.count != 0 do
	(
		sunObj.inclExclType = 1
		if coronaSLNode.excludeListIncludeMod == false then sunObj.excludeList = coronaSLNode.excludeList else sunObj.includeList = coronaSLNode.excludeList
	)
			
	theLayer = layermanager.getLayerFromName coronaSLNode.layer.name
	theLayer.addNode sunObj
	theLayer.addNode targetObj			
	
	delete coronaSLNode
	return undefined
)

fn normalizeColor col = (
	norm=if (col.r>col.g) then (if (col.r>col.b) then col.r else col.b) else (if (col.g>col.b) then col.g else col.b)
	result=color ((col.r*255.0)/norm) ((col.g*255.0)/norm) ((col.b*255.0)/norm)
	result
)

fn replaceLightNodeVRIESLight node = (
	local newLight=CreateInstance VRayIES name:(node.name+"_VRayIESLight")
	
	if (newLight!=undefined) do (
		newLight.targeted=(node.target!=undefined)
		newLight.enabled=node.on
		newLight.color_mode=0
		newLight.color=normalizeColor node.color
		newLight.rotation_X=node.xRotation
		newLight.rotation_Y=node.yRotation
		newLight.rotation_Z=node.zRotation
		newLight.use_light_shape=1
		if (node.webFile!=undefined) then (
			newLight.ies_file=node.webFile
			local powerCoeff=node.intensity/node.originalIntensity
			newLight.power=newLight.power*powerCoeff
		) else (
			newLight.power=node.intensity*4*pi
		)
		if (node.useMultiplier) do (newLight.power=(newLight.power*node.multiplier)/100.0)
		if (hasProperty newLight "override_shape") then (
			newLight.override_shape=true
			case of (
				(node.type==#Target_Point or node.type==#Free_Point): (
					newLight.shape=0
				)
				
				(node.type==#Target_Line or node.type==#Free_Line or
				node.type==#Target_Linear or node.type==#Free_Linear): (
					newLight.shape=1
					newLight.length=node.light_length
				)
				
				(node.type==#Target_Rectangle or node.type==#Free_Rectangle or
				node.type==#Target_Area or node.type==#Free_Area): (
					newLight.shape=1
					newLight.width=node.light_width
					newLight.length=node.light_length
				)
				
				(node.type==#Target_Disc or node.type==#Free_Disc): (
					newLight.shape=2
					newLight.diameter=node.light_radius*2
				)
				
				(node.type==#Target_Sphere or node.type==#Free_Sphere): (
					newLight.shape=3
					newLight.diameter=node.light_radius*2
				)
				
				(node.type==#Target_Cylinder or node.type==#Free_Cylinder): (
					newLight.shape=5
					newLight.length=node.light_length
					newLight.diameter=node.light_radius*2
				)
			)
		)
		if (node.includeList!=undefined) do (newLight.includeList=node.includeList)
		if (node.excludeList!=undefined) do (newLight.excludeList=node.excludeList)
	)
	
	node.name = (node.name+"_VRayIESLight")
	node.baseObject=newLight
	newLight
)

fn replaceLightNodeVRayLight node = (
	local newLight=CreateInstance VRayLight name:(node.name+"_VRayLight")
	
	if (newLight!=undefined) do (
		newLight.targeted=(node.target!=undefined)
		if newLight.targeted then newLight.target_distance = node.targetDistance
		newLight.on=node.on
		newLight.color_mode=0
		newLight.color=normalizeColor node.color
		newLight.normalizeColor = 1
		newLight.multiplier=node.intensity*4*pi
		newLight.invisible= not node.baseObject.Area_Light_Sampling_Custom_Attribute.mr_EnableLightShapeRendering
		newLight.texmap_on=node.projector
		if (node.projector) do newLight.texmap=node.projectorMap
		if (node.useMultiplier) do (newLight.multiplier=(newLight.multiplier*node.multiplier)/100.0)
		case of (
			(node.type==#Target_Rectangle or node.type==#Free_Rectangle or
			node.type==#Target_Area or node.type==#Free_Area): (
				newLight.type=0
				newLight.size0=node.light_width / 2.0
				newLight.size1=node.light_length / 2.0
				newLight.DoubleSided = true
			)
			
			(node.type==#Target_Disc or node.type==#Free_Disc): (
				newLight.type=4
				newLight.size0=node.light_radius
				newLight.DoubleSided = true
			)
			
			(node.type==#Target_Sphere or node.type==#Free_Sphere): (
				newLight.type=2
				newLight.size0=node.light_radius
			)
		)
		
		if (node.includeList!=undefined) do (newLight.includeList=node.includeList)
		if (node.excludeList!=undefined) do (newLight.excludeList=node.excludeList)
	)
	
	node.name = (node.name+"_VRayLight")
	node.baseObject=newLight
	newLight
)

fn replaceLight_mr_Sun lightsys = (
	if (classof lightsys.sun==mr_Sun) do (
		local mrSun_on=lightsys.sun.on
		local mrSun_skymult=lightsys.sun.skymult
		
		lightsys.sun=createInstance VRaySun
		lightsys.sun.enabled=mrSun_on
		lightsys.sun.intensity_multiplier=mrSun_skymult
		
		lightsys.sky=undefined
	)
)

fn replaceLightNode node = (
	local lightClassStandard=#(Free_Light, Target_Light,
									Free_Point, Target_Point,
									Free_Linear, Target_Linear,
									Free_Area, Target_Area,
									Free_Disc, Target_Disc,
									Free_Sphere, Target_Sphere,
									Free_Cylinder, Target_Cylinder)
									
	local lightClassCorona=#(CoronaLight, CoronaSun)
	
	local newLight		
	if (isValidNode node) do (
		if ((findItem lightClassStandard (classof node))!=0) do (
			case (node.distribution) of (
				0: newLight=replaceLightNodeVRayLight node	-- Uniform Spherical
				1: newLight=undefined						-- Spotlight
				2: newLight=replaceLightNodeVRayLight node	-- Uniform Diffuse
				3: newLight=replaceLightNodeVRIESLight node	-- Photometric Web
			)
		)
		
		local foundCL = 0
		if ((foundCL = findItem lightClassCorona (classof node))!=0) do (
			case (foundCL) of (
				1: newLight=replaceCoronaLightNodeVRayLight node	-- Uniform Spherical
				2: newLight=replaceCoronaSunLightNodeVRayLight node
			)				
		)
	)
	
	if (newLight!=undefined) do (
		print (node.name)
	)
	
	newLight!=undefined
)

fn lightsToVRay selectedOnly = (
	format "Converting scene Lights...\n"
	local oldLights=#()
	
	-- First traverse scene for daylight systems otherwise the sun/sky lights in them will appear as standalone
	-- lights to the normal lights replace traversal.
	if (vraySceneConverterRollout.check_optLights.checked) do (
		for i=1 to $helpers.count do (
			if (classof $helpers[i]==DaylightAssemblyHead) do (
				replaceLight_mr_Sun $helpers[i]
			)
		)
		
		if ((environmentMap!=undefined) and (classof environmentMap==mr_Physical_Sky)) do (
			environmentMap=VRaySky()
		)
	)
	
	if selectedOnly do  format "Converting scene Lights for selected lights...\n"
	for i=1 to $lights.count do (
		if selectedOnly then (
			if $lights[i].isSelected do
				if (replaceLightNode $lights[i]) do ( append oldLights $lights[i] )
		) else (
			if (replaceLightNode $lights[i]) do ( append oldLights $lights[i] )
		)
	)
	-- old lights need to stay until the conversion is done, because deleteing them
	-- on the run messes nodes' order during traversal
	format "Done\n"
)

fn replaceCameraNode node = (
	local cameraClassStandard=#(Freecamera, Targetcamera)
	
	local newCamera
	local newCameraName
	if (isValidNode node) do (
		local camIdx = findItem cameraClassStandard (classof node)
		if (camIdx!=0) do (
			--Implementation
			
			if (((maxVersion())[1]) >= 18000) then (
				newCamera=CreateInstance Physical
				newCameraName = node.name+"_PhysicalCamera"
			)
			else (
				newCamera=CreateInstance VRayPhysicalCamera
				newCameraName = node.name+"_VRayPhysicalCamera"
			)
			if (newCamera!=undefined) do (
				if (camIdx == 1) then (
					newCamera.targeted = off
					newCamera.target_distance = node.baseObject.targetDistance
				)
				else (
					newCamera.targeted = off
					newCamera.target_distance = node.targetDistance
					newCamera.targeted = on
				)
			
				if (node.fov.isAnimated) then(
					newCamera.specify_fov = on
					classOfNodeController = classof node.fov.controller
					--newCamera.fov.controller=bezier_float()
					newCamera.fov.controller = createInstance classOfNodeController
					nkeys = node.fov.keys
					for k in nkeys do appendKey newCamera.fov.keys k
					for i=1 to nkeys.count do (
						newCamera.fov.keys[i].value=node.fov.keys[i].value
					)
				) else (
					newCamera.specify_fov = on
					newCamera.fov = node.fov
					newCamera.specify_fov = off
				)
				
				node.baseObject=newCamera
				node.name = newCameraName
			)
		)
	)
	
	if (newCamera!=undefined) do (
		print (node.name)
	)
	
	newCamera!=undefined
)

fn camerasToVRay selectedOnly = (
	if (vraySceneConverterRollout.check_optCameras.checked) do (
		format "Converting scene Cameras...\n"
		local oldCameras=#()
		
		if selectedOnly do format "Converting scene Cameras for selected cameras only...\n"
		for i=1 to $cameras.count do (
			if selectedOnly then (
				if $cameras[i].isSelected do
					if (replaceCameraNode $cameras[i]) do ( append oldCameras $cameras[i] )
			) else (
				if (replaceCameraNode $cameras[i]) do ( append oldCameras $cameras[i] )
			)
		)
		-- old cameras need to stay until the conversion is done, because deleteing them
		-- on the run messes nodes' order during traversal
		format "Done\n"
	)
)

--Convert Corona Proxy to VRay Proxy
fn getOrigCoronaProxy  selectedOnly =  ( --get list of original proxies
	arrayOrigCoronaProxy = #()
	arrayOrigCoronaProxy_pathes = #()
	 for i in objects where (classOf i == CProxy)  and (not selectedOnly or selectedOnly and i.isSelected) do
	 (	 
		 if (findItem arrayOrigCoronaProxy_pathes  i.filename) == 0 and doesFileExist i.filename != false do
		 (
			append arrayOrigCoronaProxy i
			append arrayOrigCoronaProxy_pathes i.filename
		)
	)
	arrayOrigCoronaProxy
)

fn getData_InstProxy_byOrig ocp = ( --get list of dependent proxies
	arrayCoronaProxy = #()
	for i in objects where classOf i == CProxy do
	(
		if i.filename != undefined and i.filename == ocp.filename do append arrayCoronaProxy #(i, i.name, getFilenameFile  i.filename, getFilenamePath i.filename, i.transform, i.previzType)
	)
	arrayCoronaProxy
)	
		
fn coronaProxiesToVray thePath selectedOnly = (
	OrigCorPr = getOrigCoronaProxy selectedOnly --list of corona proxies
	if OrigCorPr.count != 0 do (
		numCP = OrigCorPr.count
		for i = 1 to numCP do (
			theData_InstCorPr = getData_InstProxy_byOrig OrigCorPr[i] -- list of proxies related to each corona poxy
			
			tempObj = instance OrigCorPr[i] wirecolor:red 
			tempObj.transform = (matrix3 [1,0,0] [0,1,0] [0,0,1] [0,0,0])
			tempObj.previzType = 3
			converttomesh tempObj
			objNF = (tempObj.numfaces/10.0) as integer -- preview 1/10 ot numfaces
			
			
			tempObj.name = theData_InstCorPr[1][3]
			select tempObj
			if thePath != undefined and thePath != "Select path please..." then (
				tempVrayPr = vrayMeshExport tempObj autoCreateProxies:true maxPreviewFaces:objNF  meshfile:(thePath + "\\" + theData_InstCorPr[1][3])
			) else (
				tempVrayPr = vrayMeshExport tempObj autoCreateProxies:true maxPreviewFaces:objNF  meshfile:(theData_InstCorPr[1][4] + theData_InstCorPr[1][3]) 
			)
			tempVrayPr[1].display = 0  
			tempVrayPr[1].visibility_lists_type = 0 
			tempVrayPr[1].visibility_list_names[1] = false
			for i in (getData_InstProxy_byOrig OrigCorPr[i]) do i[1].name = uniqueName tempObj.name
			
			--replace each corona proxy with vray proxy -> tempVrayPr
			instanceReplace OrigCorPr[i] tempVrayPr[1]
			delete tempVrayPr
		)
	)
)
		
fn ConvertToVRay selectedOnly = (
	mtlsToVRay selectedOnly
	shadowsToVRay selectedOnly
	lightsToVRay selectedOnly
	camerasToVRay selectedOnly
	coronaProxiesToVray undefined selectedOnly
)
convertToVRay=ConvertToVRay

-- Bitmap2VRayHDRI converter
convertB2HDRI_ConvertEXR=false
convertB2HDRI_ConverterPath=undefined
convertB2HDRI_ConvertHalf=false
convertB2HDRI_ConvertCompression="zip"
convertB2HDRI_ConvertTilesize=64
convertB2HDRI_ConvertLinear="auto"

convertB2HDRI_ConverterErrors=0

vrayBitmap2VRayHDRI
convertB2HDRI_TEXRConverterRollout
rollout convertB2HDRI_Rollout "V-Ray Bitmap to VRayHDRI texmap converter" width: 460 height: 110 (
	label label_message "All scene materials' Bitmap texmaps will be converted to VRayHDRI texmap." width:400 height:13 align:#center offset:[0, 10]
	
	checkbox check_convertEXR "Convert bitmaps to tiled EXR." checked:false width:400 align:#center offset:[0, 10]
	checkbox check_convertObjectBsInScene "Convert All scene Objects' Bitmap texmaps to VRayHDRI texmap." checked:false width:400 align:#center
	
	button btn_start "Convert" width:60 align:#right across:2 offset:[150, 0]
	button btn_cancel "Cancel" width:60 align:#right
	
	local cancelled=false
	local convertPath=undefined
	local convertHalf=false
	local convertCompression="zip"
	local convertTilesize=64
	local convertLinear="auto"
	
	on btn_start pressed do (
		convertB2HDRI_ConverterErrors=0
		cancelled=false
		
		if (check_convertEXR.checked) do (
			CreateDialog convertB2HDRI_TEXRConverterRollout modal:true
		)
		if (NOT cancelled) do (
			vrayBitmap2VRayHDRI exrConvert:check_convertEXR.checked exrPath:convertPath exrHalf:convertHalf exrCompress:convertCompression exrTilesize:convertTilesize exrLinear:convertLinear applyOnSceneObjects:check_convertObjectBsInScene.checked
			if (convertB2HDRI_ConverterErrors>0) do (
				messageBox ("There were "+(convertB2HDRI_ConverterErrors as String)+" images that did not convert successfully to tiled EXR format. The original images were left in place.") title:"Tiled EXR conversion error"
			)
		)
		
		destroyDialog convertB2HDRI_Rollout
	)
	
	on btn_cancel pressed do (
		destroyDialog convertB2HDRI_Rollout
	)
)

rollout ConvertB2HDRI_TEXRConverterRollout "Img2tiledexr tool conversion options" width: 460 height: 200 (
	label label_message "If the img2tiledexr tool was not found it should be set manually." width:400 height:13 align:#center offset:[0, 10]
	
	edittext edit_path width:394 across:2 align:#left offset:[13, 0]
	button btn_path "..." width:20 height:18 align:#right offset:[0, 0]
	
	checkbox check_half "Half - store data as 16-bit floating point numbers." checked:false width:400 align:#center offset:[0, 10]
	
	label label_compression "Compression method:" width:200 height:13 across:2 align:#center offset:[10, 2]
	dropdownlist drop_compression items:#("None", "RLE", "Zips", "Zip", "PIZ", "PXR24") selection:4 width:100 align:#center offset:[-150, 0]
	
	label label_tilesize "Tile size:" width:200 height:13 across:2 align:#center offset:[10, 2]
	dropdownlist drop_tilesize items:#("32x32", "64x64", "128x128") selection:2 width:100 align:#center offset:[-150, 0]
	
	label label_linearmode "Linear mode:" width:200 height:13 across:2 align:#center offset:[10, 2]
	dropdownlist drop_linearmode items:#("Off", "On", "Auto") selection:3 width:100 align:#center offset:[-150, 0]
	
	button btn_set "OK" width:60 align:#right across:2 offset:[150, 0]
	button btn_cancel "Cancel" width:60 align:#right
	
	on ConvertB2HDRI_TEXRConverterRollout open do (
		if (convertB2HDRI_Rollout.check_convertEXR.checked) do (
			local ver=getMaxReleaseAsString()
			local tmpPath=getVRayInstallPath ver
			if (tmpPath!=undefined) do (
				append tmpPath "\\tools\\img2tiledexr.exe"
				edit_path.text=tmpPath
				edit_path.enabled=false
				btn_path.enabled=false
			)
		)
	)
	
	on btn_path pressed do (
		local toolPath=getOpenFileName filename:edit_path.text \
								types:"Executable File (img2tiledexr.exe)|img2tiledexr.exe"
		if (toolPath!=undefined) do (
			edit_path.text=toolPath
		)
	)
	
	on btn_set pressed do (
		convertB2HDRI_Rollout.convertPath=edit_path.text
		convertB2HDRI_Rollout.convertHalf=check_half.checked
		convertB2HDRI_Rollout.convertCompression=case drop_compression.selection of (
			1:	"none"
			2:	"rle"
			3:	"zips"
			4:	"zip"
			5:	"piz"
			6:	"pxr24"
		)
		convertB2HDRI_Rollout.convertTilesize=case drop_tilesize.selection of (
			1:	32
			2:	64
			3:	128
		)
		convertB2HDRI_Rollout.convertLinear=case drop_linearmode.selection of (
			1:	"off"
			2:	"on"
			3:	"auto"
		)
		destroyDialog convertB2HDRI_TEXRConverterRollout
	)
	
	on btn_cancel pressed do (
		convertB2HDRI_Rollout.cancelled=true
		destroyDialog convertB2HDRI_TEXRConverterRollout
	)
)
convertB2HDRI_TEXRConverterRollout=ConvertB2HDRI_TEXRConverterRollout

fn convertB2HDRI_img2tiledexr imgpath=(
	if ((classof imgpath)==String) then (
		if ((stricmp (getFilenameType imgpath) ".exr")!=0) then (
			outpath=(getFilenamePath imgpath)+(getFilenameFile imgpath)+".exr"
		)
		else (
			outpath=imgpath
		)
		
		if ((classof convertB2HDRI_ConverterPath)==String) do (
			local cmdConverter=("\""+convertB2HDRI_ConverterPath+"\" \""+imgpath+"\" \""+outpath+"\"")
			if (convertB2HDRI_ConvertHalf) then (append cmdConverter " -half")
			else (append cmdConverter " -32bit")
			append cmdConverter (" -compression "+convertB2HDRI_ConvertCompression)
			append cmdConverter (" -tileSize "+(convertB2HDRI_ConvertTilesize as String))
			append cmdConverter (" -linear "+convertB2HDRI_ConvertLinear)
			
			vrayShellLaunch cmdConverter waitExec:true hideConsole:true
		)
		
		if (doesFileExist outpath) then (
			outpath
		)
		else (
			if (convertB2HDRI_ConverterErrors!=undefined) do (
				convertB2HDRI_ConverterErrors=convertB2HDRI_ConverterErrors+1
			)
			imgpath
		)
	)
	else (
		undefined
	)
)

fn convertB2HDRI_Texmap texmap=(
	if (texmap!=undefined and (classof texmap)==Bitmaptexture) then (
		local tex=VRayHDRI()
		tex.name=texmap.name+"_VRay"
		
		if (texmap.fileName != undefined) do (
			if (convertB2HDRI_ConvertEXR) then (
				tex.HDRIMapName=convertB2HDRI_img2tiledexr texmap.fileName
				
				if (tex.HDRIMapName!=undefined) do (
					local isHDRI=(((stricmp (getFilenameType texmap.fileName) ".exr")==0) OR ((stricmp (getFilenameType texmap.fileName) ".hdr")==0))
					if (NOT isHDRI) do (
						local gamma=vrayGetBitmapGamma texmap
						if (gamma!=undefined) then (
							if (convertB2HDRI_ConvertLinear!="off") then (
								gamma=gamma/2.2
							)
							tex.gamma=1.0/gamma
						)
					)
				)
			)
			else (
				tex.HDRIMapName=texmap.fileName
				local isHDRI=(((stricmp (getFilenameType texmap.fileName) ".exr")==0) OR ((stricmp (getFilenameType texmap.fileName) ".hdr")==0))
				if (NOT isHDRI) do (
					local gamma=vrayGetBitmapGamma texmap
					if (gamma!=undefined) then (
						tex.gamma=1.0/gamma
					)
				)
			)
		)
		
		tex.mapType=4
		tex.UVGen.blur=texmap.coords.blur
		tex.UVGen.mapping=texmap.coords.mapping
		tex.UVGen.mapChannel=texmap.coords.mapChannel
		tex.UVGen.mappingType=texmap.coords.mappingType
		tex.UVGen.UVW_Type=texmap.coords.UVW_Type
		tex.UVGen.U_Mirror=texmap.coords.U_Mirror
		tex.UVGen.V_Mirror=texmap.coords.V_Mirror
		tex.UVGen.U_Tile=texmap.coords.U_Tile
		tex.UVGen.V_Tile=texmap.coords.V_Tile
		tex.UVGen.showMapOnBack=texmap.coords.showMapOnBack
		tex.UVGen.Noise_On=texmap.coords.Noise_On
		tex.UVGen.Noise_Animate=texmap.coords.Noise_Animate
		tex.UVGen.realWorldScale=texmap.coords.realWorldScale
		tex.UVGen.realWorldHeight=texmap.coords.realWorldHeight
		tex.UVGen.realWorldWidth=texmap.coords.realWorldWidth
		tex.UVGen.phase=texmap.coords.phase
		tex.UVGen.U_Angle=texmap.coords.U_Angle
		tex.UVGen.V_Angle=texmap.coords.V_Angle
		tex.UVGen.W_Angle=texmap.coords.W_Angle
		tex.UVGen.Noise_Amount=texmap.coords.Noise_Amount
		tex.UVGen.Noise_Size=texmap.coords.Noise_Size
		tex.UVGen.Noise_Levels=texmap.coords.Noise_Levels
		tex.UVGen.Blur_Offset=texmap.coords.Blur_Offset
		tex.UVGen.u_tiling=texmap.coords.u_tiling
		tex.UVGen.v_tiling=texmap.coords.v_tiling
		tex.UVGen.u_offset=texmap.coords.u_offset
		tex.UVGen.v_offset=texmap.coords.v_offset
		
		--Convert crop/placement
		tex.cropplace_on = texmap.apply
		tex.cropplace_mode = texmap.cropPlace
		tex.cropplace_u = texmap.clipu
		tex.cropplace_v = texmap.clipv
		tex.cropplace_width = texmap.clipw
		tex.cropplace_height = texmap.cliph
		
		--Convert output parameters including curve control
		replaceInstances tex.output (copy texmap.output)
		
		tex
	)
	else (
		undefined
	)
)

fn convertB2HDRI_SubTex texmap = (
	if (texmap!=undefined) then (
		-- check if we have already converted this texture; if yes, just return
		-- the already converted one	
		local i=findItem orig_bitmaptextures texmap
		if i>0 then return new_hdritexmaps[i]
		
		for i = 1 to (getNumSubTexmaps texmap) do (
			local tex=(convertB2HDRI_SubTex (getSubTexmap texmap i))
			if (tex!=undefined) do (
				setSubTexmap texmap i tex
			)
		)
		
		local new_tex = texmap
		if ((classof texmap)==Bitmaptexture) then (
			new_tex=convertB2HDRI_Texmap texmap
		)
		
		-- if converted, add to the list of converted textures
		if new_tex!=texmap then (
			append orig_bitmaptextures texmap
			append new_hdritexmaps new_tex
			return new_tex
		)
	)
	return undefined
)

fn convertB2HDRI_SubMtl material = (
	if (material!=undefined) do (
		for i=1 to (getNumSubMtls material) do (
			convertB2HDRI_SubMtl (getSubMtl material i)
		)
		
		for i=1 to (getNumSubTexmaps material) do (
			local tex=(convertB2HDRI_SubTex (getSubTexmap material i))
			if (tex!=undefined) do (setSubTexmap material i tex)
		)
	)
)

fn convertB2HDRI_ArrayParameter texmapArray = (
	if ((classof texmapArray)==ArrayParameter) then (
		for i=1 to texmapArray.count do (
			if (texmapArray[i]!=undefined) do (
				if ((superclassof texmapArray[i])==textureMap) then (
					hdriTex=convertB2HDRI_SubTex texmapArray[i]
					if (hdriTex!=undefined) do texmapArray[i]=hdriTex
				) 
			)
		)
	)
)

fn convertB2HDRI_Objects obj = (
	if(obj != undefined) do (
		local specificObj
		local classOfObj = classof obj
		if ((superclassof obj)==light) then (
			if (classOfObj == miAreaLightOmni) then (
				specificObj = obj.Omni_Light
			)
			else if (classOfObj == miAreaLight) then (
				specificObj = obj.Target_Spot
			)
			else (
				specificObj = obj
			)
		)
		else (
			specificObj = obj
		)
		
		local listPropNames = getPropNames specificObj
		
		local hdriTex = undefined
		local propValue = undefined
		
		for propName in listPropNames do (
			if (hasProperty specificObj propName) then (
				propValue = getProperty specificObj propName
				if ((superclassof propValue)==textureMap) then (
					hdriTex = convertB2HDRI_SubTex propValue
					if (hdriTex!=undefined) do setProperty specificObj propName hdriTex
				) else if((superclassof propValue)==material) then (
					convertB2HDRI_SubMtl propValue
				) else convertB2HDRI_ArrayParameter propValue
			)
		) 
	)
)

fn convertB2HDRI_Node node applyOnObjects = (
	if isValidNode node then (
		convertB2HDRI_SubMtl node.material
		if (applyOnObjects) do (convertB2HDRI_Objects node.baseObject)
		--convert bitmaps of node modifiers
		for imod in node.modifiers do (
			convertB2HDRI_Objects imod
		)
	)
	for i=1 to node.children.count do (
		convertB2HDRI_Node node.children[i] applyOnObjects
	)
)

fn convertB2HDRI_RenderSettings = (
	if(checkProductRenderer()) then (
		productRenderer = renderers.production
		listRenderSettingsNames = getPropNames productRenderer
		for rParamName in listRenderSettingsNames do (
			if (hasProperty productRenderer rParamName) then (
				rParam = getProperty productRenderer rParamName
				if ((superclassof rParam)==textureMap) then (
					hdriTex = convertB2HDRI_SubTex rParam
					if (hdriTex!=undefined) do setProperty productRenderer rParamName hdriTex
				) else if ((superclassof rParam)==material) then (
					convertB2HDRI_SubMtl rParam
				)
			)
		)
	)
)

fn convertB2HDRI_Environment = (
	-- Convert Environment map
	local envtex=convertB2HDRI_SubTex environmentMap
	if (envtex!=undefined) do environmentMap=envtex
	
	-- Convert Atmosphere effects
	for iAtmos=1 to numAtmospherics do (
		currAtmospheric = getAtmospheric iAtmos
		listAtmosNames = getPropNames currAtmospheric
		
		for atmosName in listAtmosNames do (
			if (hasProperty currAtmospheric atmosName) then (
				atmosParam = getProperty currAtmospheric atmosName
				if ((superclassof atmosParam)==textureMap) then (
					hdriTex = convertB2HDRI_SubTex atmosParam
					if (hdriTex!=undefined) do setProperty currAtmospheric atmosName hdriTex
				)
			)
		)
	)
	
	-- Convert Environment effects
	for iEffect=1 to numEffects do (
		currEffect = getEffect iEffect
		listEffectNames = getPropNames currEffect
		for effectName in listEffectNames do (
			if (hasProperty currEffect effectName) then (
				effectParam = getProperty currEffect effectName
				if ((superclassof effectParam)==textureMap) then (
					hdriTex = convertB2HDRI_SubTex effectParam
					if (hdriTex!=undefined) do setProperty currEffect effectName hdriTex
				)
			)
		)
	)
)

fn VrayBitmap2VRayHDRI exrConvert: exrPath: exrHalf: exrCompress: exrTilesize: exrLinear: applyOnSceneObjects: =(
	convertB2HDRI_ConvertEXR=false
	if (exrConvert!=unsupplied) do (
		if ((classof exrConvert)!=BooleanClass) do (print "Expected Boolean for exrConvert:")
		if (exrConvert) do (
			if (exrPath!=unsupplied) then (
				if ((classof exrPath)!=String) do (print "Expected String for exrPath:")
				convertB2HDRI_ConverterPath=exrPath
			)
			else (
				if (convertB2HDRI_ConverterPath==undefined) do (
					local ver=getMaxReleaseAsString()
					convertB2HDRI_ConverterPath=getVRayInstallPath ver
					if (convertB2HDRI_ConverterPath!=undefined) do (append convertB2HDRI_ConverterPath "\\tools\\img2tiledexr.exe")
				)
			)
		)
		convertB2HDRI_ConvertEXR=exrConvert
	)
	
	if (exrHalf!=unsupplied) do (
		if ((classof exrHalf)!=BooleanClass) do (print "Expected Boolean for exrHalf:")
		convertB2HDRI_ConvertHalf=exrHalf
	)
	if (exrCompress!=unsupplied) do (
		if ((classof exrCompress)!=String) do (print "Expected String for exrCompress:")
		convertB2HDRI_ConvertCompression=exrCompress
	)
	if (exrTilesize!=unsupplied) do (
		if ((classof exrTilesize)!=Integer) do (print "Expected Integer for exrTilesize:")
		convertB2HDRI_ConvertTilesize=exrTilesize
	)
	if (exrLinear!=unsupplied) do (
		if ((classof exrLinear)!=String) do (print "Expected String for exrLinear:")
		convertB2HDRI_ConvertLinear=exrLinear
	)
	
	orig_bitmaptextures=#()
	new_hdritexmaps=#()
	
	local applyOnObjects=false
	if (applyOnSceneObjects!=unsupplied) do (
		applyOnObjects = applyOnSceneObjects
	)
	convertB2HDRI_Node rootNode applyOnObjects
	
	for i=1 to meditMaterials.count do (
		local m=meditMaterials[i]
		if (m.superClassID==3072) then ( -- This is a material
			convertB2HDRI_SubMtl m
		) else ( -- This is a texture
			m=convertB2HDRI_SubTex m
			if (m!=undefined) then meditMaterials[i]=m
		)
		updateMTLInMedit meditMaterials[i]
	)
	
	convertB2HDRI_RenderSettings()
	convertB2HDRI_Environment()
)
vrayBitmap2VRayHDRI=VrayBitmap2VRayHDRI

macroScript VRayProperties category:"VRay" buttontext:"V-Ray properties" tooltip:"Displays the VRay object or light properties" (
	doVRayObjectProperties()
)

macroScript VRaySceneConverter category:"VRay" buttontext:"V-Ray scene converter" tooltip:"Convert all scene materials to VRayMtl and all raytraced shadows to VRay shadows" (
	CreateDialog vraySceneConverterRollout modal:true
)

macroScript VRayBitmap2VRayHDRITexmapConverter
	category:"VRay"
	buttonText:"V-Ray Bitmap to VRayHDRI converter"
	tooltip:"Convert all scene materials' Bitmaps to VRayHDRI"
(
	CreateDialog convertB2HDRI_Rollout modal:true
)

macroScript VRayMeshExport category:"VRay" buttontext:"V-Ray mesh export" tooltip:"Exports the selected meshes to an external file" (
	doVRayMeshExport()
)

macroScript VRayShowVFB category:"VRay" buttontext:"V-Ray VFB" tooltip:"Displays the last rendered V-Ray VFB, if is exists" (
	r=renderers.production
	cls=r.classid
	if (cls[1]==1941615238 and cls[2]==2012806412) then r.showLastVFB()
	if (cls[1]==1770671000 and cls[2]==1323107829) do r.V_Ray_settings.showLastVFB()
)

-- Menu bar V-Ray action items
macroScript VRayProxy
	category:"VRay"
	buttonText:"V-Ray Proxy"
	toolTip: "Create V-Ray Proxy"
(
	local enabled = false
	
	on execute do StartObjectCreation VRayProxy
	
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayFur
	category:"VRay"
	buttonText:"V-Ray Fur"
	toolTip: "Create V-Ray Fur"
(
	fn setSourceNode obj = obj.sourceNode = $
	
	local enabled = false
	
	on execute do (
		if($ == undefined) then warningToSelectGeometry "V-Ray Fur"
		
		local vrFur = undefined
		if(superClassOf $ == GeometryClass) then (
			vrFur = VRayFur sourceNode:$ pos:$.pos
			append $.children vrFur
			select vrFur
			setCommandPanelTaskMode #modify
		) else if(isKindOf $ ObjectSet) then (
			for obj in $ do (
				if(superClassOf obj == GeometryClass) do (
					vrFur = VRayFur sourceNode:obj pos:obj.pos
					append obj.children vrFur
				)
			)
			select vrFur
			setCommandPanelTaskMode #modify
		)
		else (
			print "Please select a geometry object to apply V-Ray Fur!"
		)
	)
	
	on isEnabled do (
		enabled=checkProductRenderer()
		enabled AND (superClassOf $ == GeometryClass)
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayPlane
	category:"VRay"
	buttonText:"V-Ray Plane"
	toolTip: "Create V-Ray Plane"
(
	local enabled = false
	
	on execute do StartObjectCreation VRayPlane
	
	on isEnabled do (
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRaySphere
	category:"VRay"
	buttonText:"V-Ray Sphere"
	toolTip: "Create V-Ray Sphere"
(
	local enabled = false
	
	on execute do StartObjectCreation VRaySphere
	
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayMetaball
	category:"VRay"
	buttonText:"V-Ray Metaball"
	toolTip: "Create V-Ray Metaball"
(
	local enabled = false
	
	on execute do StartObjectCreation VRayMetaball
	
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayClipper
	category:"VRay"
	buttonText:"V-Ray Clipper"
	toolTip: "Create V-Ray Clipper"
(
	local enabled = false
	
	on execute do StartObjectCreation VRayClipper
	
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayInstancer
	category:"VRay"
	buttonText:"V-Ray Instancer"
	toolTip: "Create V-Ray Instancer"
(
	local enabled = false
	
	on execute do StartObjectCreation VRayInstancer
	
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayVolumeGrid
	category:"VRay"
	buttonText:"V-Ray VolumeGrid"
	toolTip: "Create V-Ray VolumeGrid"
(
	local enabled = false
	
	on execute do StartObjectCreation VRayVolumeGrid
	
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayLight_Plane
	category:"VRay"
	buttonText:"V-Ray Plane Light"
	toolTip: "Create V-Ray Plane Light"
(
	fn setType obj = (if (classof(obj)==VRayLight) do (obj.type = 0))
	
	local enabled = false
	
	on execute do StartObjectCreation VRayLight newNodeCallback: setType
	
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayLight_Dome
	category:"VRay"
	buttonText:"V-Ray Dome Light"
	toolTip: "Create V-Ray Dome Light"
(
	fn setType obj = (if (classof(obj)==VRayLight) do (obj.type = 1))
	
	local enabled = false
	
	on execute do StartObjectCreation VRayLight newNodeCallback: setType
		
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayLight_Sphere
	category:"VRay"
	buttonText:"V-Ray Sphere Light"
	toolTip: "Create V-Ray Sphere Light"
(
	fn setType obj = (if (classof(obj)==VRayLight) do (obj.type = 2))
	
	local enabled = false
	
	on execute do StartObjectCreation VRayLight newNodeCallback: setType
		
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayLight_Mesh
	category:"VRay"
	buttonText:"V-Ray Mesh Light"
	toolTip: "Create V-Ray Mesh Light"
(
	fn setType obj = (if (classof(obj)==VRayLight) do (obj.type = 3))
	
	local enabled = false
	
	on execute do StartObjectCreation VRayLight newNodeCallback: setType
		
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayIES
	category:"VRay"
	buttonText:"V-Ray IES Light"
	toolTip: "Create V-Ray IES Light"
(
	local enabled = false
	
	on execute do StartObjectCreation VRayIES
		
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayAmbientLight
	category:"VRay"
	buttonText:"V-Ray Ambient Light"
	toolTip: "Create V-Ray Ambient Light"
(
	local enabled = false
	
	on execute do StartObjectCreation VRayAmbientLight
		
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRaySun
	category:"VRay"
	buttonText:"V-Ray Sun"
	toolTip: "Create V-Ray Sun"
(
	local enabled = false
	
	on execute do StartObjectCreation VRaySun
		
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayDomeCamera
	category:"VRay"
	buttonText:"V-Ray Dome Camera"
	toolTip: "Create V-Ray Dome Camera"
(
	local enabled = false
	
	on execute do StartObjectCreation VRayDomeCamera
		
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayPhysicalCamera
	category:"VRay"
	buttonText:"V-Ray Physical Camera"
	toolTip: "Create V-Ray Physical Camera"
(
	local enabled = false
	
	on execute do StartObjectCreation VRayPhysicalCamera
		
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayLightMeter
	category:"VRay"
	buttonText:"V-Ray Light Meter"
	toolTip: "Create V-Ray Light Meter"
(
	local enabled = false
	
	on execute do StartObjectCreation VRayLightMeter
		
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayBPTracer
	category:"VRay"
	buttonText:"V-Ray BPTracer"
	toolTip: "Create V-Ray BPTracer"
(
	local enabled = false
	
	on execute do StartObjectCreation VRayBPTracer
		
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayStereoscopic
	category:"VRay"
	buttonText:"V-Ray Stereoscopic"
	toolTip: "Create V-Ray Stereoscopic"
(
	local enabled = false
	
	on execute do StartObjectCreation VRayStereoscopic
		
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayStereoRig
	category:"VRay"
	buttonText:"V-Ray StereoRig"
	toolTip: "Create V-Ray StereoRig"
(
	local enabled = false
	
	on execute do StartObjectCreation VRayStereoRig
		
	on isEnabled do
	(
		enabled=checkProductRenderer()
		enabled
	)
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroscript VRayDisplacementMod
	category:"VRay"
	buttonText:"V-Ray Displacement Mod"
	toolTip: "V-Ray Displacement Mod"
(
	local enabled = false
	
	on execute do (
		if($ == undefined) then warningToSelectGeometry "Displacement Modifier"
		
		if(superClassOf $ == GeometryClass) then (
			if (ValidModifier $ VRayDisplacementMod) do (
				addModifier $ (VRayDisplacementMod())
			)
		) else if(isKindOf $ ObjectSet) then (
			for obj in $ do (
				if ((superClassOf obj == GeometryClass) AND (ValidModifier obj VRayDisplacementMod)) do (
					addModifier obj (VRayDisplacementMod())
				)
			)
		)
		else (
			print "Please select a geometry object to apply Displacement Modifier!"
		)
	)
	
	on isEnabled return ValidModifier $ VRayDisplacementMod
	
	on isVisible do
	(
		checkRenderer()
		enabled
	)
)

macroscript VRayOrnatrixMod
	category:"VRay"
	buttonText:"V-Ray Ornatrix Mod"
	toolTip: "V-Ray Ornatrix Mod"
(
	local enabled = false
	
	on execute do 
	(
		if(superClassOf $ == GeometryClass) then (
			if (ValidModifier $ VRayOrnatrixMod) do (
				addModifier $ (VRayOrnatrixMod())
			)
		) else if(isKindOf $ ObjectSet) then (
			for obj in $ do (
				if ((superClassOf obj == GeometryClass) AND (ValidModifier obj VRayOrnatrixMod)) do (
					addModifier obj (VRayOrnatrixMod())
				)
			)
		)
		else (
			warningToSelectGeometry "Ornatrix Modifier"
		)
	)
	
	on isEnabled return ValidModifier $ VRayOrnatrixMod
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroscript VRayHairFarmMod
	category:"VRay"
	buttonText:"V-Ray HairFarm Mod"
	toolTip: "V-Ray HairFarm Mod"
(
	local enabled = false
	
	on execute do (
		if($ == undefined) then warningToSelectGeometry "V-Ray HairFarm"
		
		if(superClassOf $ == GeometryClass) then (
			if (ValidModifier $ VRayHairFarmMod) do (
				addModifier $ (VRayHairFarmMod())
			)
		) else if(isKindOf $ ObjectSet) then (
			for obj in $ do (
				if ((superClassOf obj == GeometryClass) AND (ValidModifier obj VRayHairFarmMod)) do (
					addModifier obj (VRayHairFarmMod())
				)
			)
		)
		else (
			print "Please select a geometry object to apply V-Ray HairFarm!"
		)
	)
	
	on isEnabled return ValidModifier $ VRayHairFarmMod
	
	on isVisible do
	(
		enabled=checkProductRenderer()
		enabled
	)
)

macroScript VRayRTAnimPreview
enabledIn:#("max")
category:"VRay"
buttontext:"Make V-Ray RT Animation Preview"
tooltip:"Render an animation preview with V-Ray RT"
(
	local enabled = false
	local running = false
	local asRenderer = undefined
	local cFrame
	local saveFilePath = (GetDir #renderoutput) + "\frame"
	local saveFileExt = ".png"
	local dialogPos
	local startFrame = (animationRange.start as Integer)/160
	local endFrame = (animationRange.end as Integer)/160
	
	rollout SettingsRollout "V-Ray RT Animation Preview Render Settings" width:262 height:216 (
		label startFrameSpinLabel "Start frame:" pos:[16, 10] width:160 height:16
		spinner startFrameSpin "" pos:[175, 10] width:70 height:16 range:[0, animationRange.end, startFrame] type:#integer scale:1
		
		label endFrameSpinLabel "End frame:" pos:[16, 32] width:160 height:16
		spinner endFrameSpin "" pos:[175, 32] width:70 height:16 range:[0, animationRange.end, endFrame] type:#integer scale:1
		
		label timeLimitSpinLabel "Time limit (min.):" pos:[16, 54] width:160 height:16
		spinner timeLimitSpin "" pos:[175, 54] width:70 height:16 range:[0.0, 525600.0, asRenderer.max_render_time] type:#float scale:0.25
		
		label noiseThrLimitSpinLabel "Noise threshold:" pos:[16, 76] width:160 height:16
		spinner noiseThrLimitSpin "" pos:[175, 76] width:70 height:16 range:[0.00001, 0.99, asRenderer.aa_threshold] type:#float scale:0.01
		
		label maxPathsLimitSpinLabel "Max. paths per pixel:" pos:[16, 98] width:160 height:16
		spinner maxPathsLimitSpin "" pos:[175, 98] width:70 height:16 range:[0, 65536, asRenderer.max_paths_per_pixel] type:#integer scale:1
		
		edittext saveFileNameEdit "" text:(saveFilePath + saveFileExt) pos:[12, 120] width:208 height:17
		button saveFileNameBtn "..." pos:[226, 120] width:20 height:18

		button startPreviewBtn "Start Preview Render" pos:[16, 142] width:230 height:18
		
		button cancelPreview "Cancel" pos:[16, 164] width:230 height:18

		progressBar animProgress pos:[16, 186] width:230 height:16
		
		on startFrameSpin changed val do (
			startFrame = val
		)
		
		on endFrameSpin changed val do (
			endFrame = val
		)
		
		on timeLimitSpin changed val do (
			if (enabled) do (
				asRenderer.max_render_time = val
			)
		)
		
		on noiseThrLimitSpin changed val do (
			if (enabled) do (
				asRenderer.aa_threshold = val
			)
		)

		on maxPathsLimitSpin changed val do (
			if (enabled) do (
				asRenderer.max_paths_per_pixel = val
			)
		)
		
		on saveFileNameEdit changed val do (
			saveFilePath = (getFilenamePath val) + (getFilenameFile val)
			saveFileExt = (getFilenameType val)
		)
		
		on saveFileNameBtn pressed do (
			local newFilePath = getSaveFileName filename:saveFileNameEdit.text \
											types:"PNG Image File (*.png)|*.png|JPEG File (*.jpg)|*.jpg" \
											historyCategory:"MAXScriptFileOpenSave"
			if (newFilePath != undefined) do (
				saveFileNameEdit.text = newFilePath
				saveFilePath = (getFilenamePath newFilePath) + (getFilenameFile newFilePath)
				saveFileExt = (getFilenameType newFilePath)
			)
		)
		
		on startPreviewBtn pressed do (
			if (running == false) do (
				running = true
				startPreviewBtn.enabled = false
				animProgress.value = 0
				cFrame = startFrame
				saveFilePath = (getFilenamePath saveFileNameEdit.text) + (getFilenameFile saveFileNameEdit.text)
				saveFileExt = (getFilenameType saveFileNameEdit.text)
				
				if (vrayGetRTBitmap() != undefined) then (
					sliderTime = cFrame
					actionMan.executeAction 369690881 "1"	-- reinitialize renderer
				)
				else (
					sliderTime = cFrame
					actionMan.executeAction 0 "40701"	-- start VrayRT ActiveShade
				)
			)
		)
		
		on cancelPreview pressed do (
			startPreviewBtn.enabled = true
			running = false
		)
		
		on SettingsRollout open do (
			if (startFrame < animationRange.start) do (
				startFrame = animationRange.start
			)
			
			if (endFrame > animationRange.end) do (
				endFrame = animationRange.end
			)
		)
		
		on SettingsRollout moved pos do (
			dialogPos = pos
		)
		
		on SettingsRollout close do (
			running = false
		)
	)
	
	fn checkRenderer = (
		asRenderer = renderers.activeShade
		
		if (asRenderer != undefined) then (
			enabled = ((renderers.activeShade.classid[1] == 1770671000) and (renderers.activeShade.classid[2] == 1323107829))
		) else (
			enabled = false
		)
	)
	
	on execute do (
		checkRenderer()
		
		if (enabled) do (
			if (dialogPos != undefined) then (
				CreateDialog SettingsRollout style:#(#style_titlebar, #style_border, #style_sysmenu) pos:dialogPos modal:true
			) else (
				CreateDialog SettingsRollout style:#(#style_titlebar, #style_border, #style_sysmenu) modal:true
			)
		)
	)
	
	on isEnabled do (
		checkRenderer()
		enabled
	)
	
	on isVisible do (
		checkRenderer()
		enabled
	)

	-- Export function through global variable declaration
	global vrayRTImageComplete
	fn vrayRTImageComplete reason = (
		if (running) do (
			if ((saveFilePath != "") AND (saveFileExt != "")) do (
				local b = vrayGetRTBitmap()
				if (b != undefined) do (
					b.filename = (saveFilePath + (formattedPrint (cFrame) format:"04i") + saveFileExt)
					save b
					close b
				)
			)
			
			SettingsRollout.animProgress.value = ((((cFrame - startFrame)+1)*100)/((endFrame - startFrame)+1))
			
			if (cFrame < endFrame) then (
				cFrame += 1
				sliderTime = cFrame
				actionMan.executeAction 369690881 "1"	-- reinitialize renderer
			) else (
				SettingsRollout.startPreviewBtn.enabled = true
				running = false
			)
		)
	)
)

-- Localized menu names
menuName_Tools=#("Ferramentas", "Tools", "Extras", "Outils", "ツール(T)", "도구(T)", "工具(T)")
menuName_Tools_LightLister=#("Criador de lista de luz...", "Light Lister...", "Lichtliste...", "Liste de lumières...", "ライト リスト...", "라이트 리스트자...", "灯光列表...")
menuName_Create=#("Criar", "Create", "Erstellen", "Créer", "作成(C)", "만들기(C)", "创建(C)")
menuName_Create_Lights=#("Luzes", "Lights", "Lichtquellen", "Lumières", "ライト(L)", "라이트(L)", "灯光(L)")
menuName_Create_Cameras=#("Câmeras", "Cameras", "Kameras", "Caméras", "カメラ(C)", "카메라(C)", "摄影机(C)")
menuName_Create_Helpers=#("Ajudas", "Helpers", "Helfer", "Assistants", "ヘルパー", "도우미", "辅助对象")
menuName_Create_Systems=#("Sistemas", "Systems", "Systèmes", "Systeme", "システム", "시스템", "系统")
menuName_Modifiers=#("Modificadores", "Modifiers", "Modifikatoren", "Modificateurs", "モディファイヤ", "수정자", "修改器")
mainMenuBar_Rendering=#("Renderização", "Rendering", "Rendern", "Rendu", "レンダリング(R)", "렌더링(R)", "渲染(R)")

menuName_Enhanced_LightingCameras_Tools=#("Ferramentas (iluminação/conjunto de câmeras)", "Tools (Lighting/Cameras Set)","Hilfsmittel (Beleuchtung/Kamera-Satz)","Outils (Eclairage/caméras)","ツール (ライト/カメラ セット)","도구(조명/카메라 세트)","工具(照明/摄影机设置)")
menuName_Enhanced_Rendering_Preview=#("Captura de tela", "Screen Capture","Bildschirmausdruck","Capture d'écran","スクリーン キャプチャ","화면 캡처","屏幕捕获")
menuName_Enhanced_Objects=#("Objetos", "Objects","Objekte","Objets","オブジェクト(&O)","오브젝트(&O)","对象(&O)")
menuName_Enhanced_LightingCameras=#("Iluminação/câmeras", "Lighting/Cameras","Beleuchtung/Kameras","Eclairage/Caméras","ライト/カメラ(&L)","조명/카메라(&L)","照明/摄影机(&L)")
menuName_Enhanced_Materials=#("Materiais", "Materials", "Materialien", "Matériaux", "マテリアル(&T)", "재질(&T)", "材质(&T)")
menuName_Enhanced_Materials_Tools=#("Ferramentas (conjunto de materiais)", "Tools (Material Set)", "Hilfsmittel (Materialsatz)", "Outils (Matériaux)", "ツール (マテリアル セット)(&T)", "도구(재질 세트)(&T)", "工具(材质设置)(&T)")

-- Menu editing utility functions
-- Append menu item to a menu object;
-- Parameters:	Menu
--		Item's MacroScript Name,
--		Item's MacroScript Category,
fn appendMenuItem mobj msname mscat = mobj.addItem (menuMan.createActionItem msname mscat) -1

-- Insert menu item to a menu object at specified position;
-- Parameters:	Menu
--		Item's MacroScript Name,
--		Item's MacroScript Category,
-- 		Item's insert position
fn InsertMenuItem mobj msname mscat idx = mobj.addItem (menuMan.createActionItem msname mscat) idx

-- Insert submenu item:
-- Parameters:	Menu
--		Menu Name
--		Submenu Object
--		Menu Index
fn insertSubmenuItem mobj mname sobj mindex = (
	local subMenuItem = menuMan.createSubMenuItem mname sobj
	subMenuItem.setUseCustomTitle true
	mobj.addItem subMenuItem mindex
)

-- Return index of the first separator in a menu, if no separator last element's index is returned
-- Parameters:	Menu
fn findFirstSeparatorIdx mobj = (
	local i = 1
	while ((i <= mobj.numItems()) and (not((mobj.getItem i).getIsSeparator()))) do (
		i += 1
	)
	
	i
)

fn findLastSeparatorIdx mobj = (
	local i = mobj.numItems()
	while ((i <= 1) and (not((mobj.getItem i).getIsSeparator()))) do (
		i -= 1
	)
	
	i
)

-- Return menu item from a menu, found by title
-- Parameters:	Menu
--		Item Name
fn findMenuItem mobj iname = (
	local item = undefined
	local i
	for i = 1 to mobj.numItems() do (
		local titleElements = filterString ((mobj.getItem i).getTitle()) "&"
		local title = ""
		local j
		for j = 1 to titleElements.count do ( 
			title = append title titleElements[j]
		)
		
		if (title == iname) do (
			item = mobj.getItem i
		)
	)
	
	item
)

-- Return menu item's index from a menu, found by title
-- Parameters:	Menu
--		Item Name
fn findMenuItemIdx mobj iname = (
	local item = undefined
	local i
	for i = 1 to mobj.numItems() do (
		local titleElements = filterString ((mobj.getItem i).getTitle()) "&"
		local title = ""
		local j
		for j = 1 to titleElements.count do ( 
			title = append title titleElements[j]
		)
		
		if (title == iname) do (
			item = i
		)
	)
	
	item
)

-- Register V-Ray objects in menu bar
fn registerVRayMenuBarItems = (
	local updateEnable = false
	local subMenu -- temporary submenu object
	local menuIndex -- temporary index used for item positioning calculation
	
	local mainMenuBar = menuMan.getMainMenuBar()
	
	local toolsMenuItem
	for i=1 to menuName_Tools.count do (if (toolsMenuItem==undefined) do (toolsMenuItem=findMenuItem mainMenuBar menuName_Tools[i]))
	if (toolsMenuItem != undefined) do (
		local toolsMenu = toolsMenuItem.getSubMenu()
		
		-- Add Light Lister item
		if ((findMenuItem toolsMenu "V-Ray Light Lister") == undefined) do (
			local lightListerIndex
			for i=1 to menuName_Tools_LightLister.count do (if (lightListerIndex==undefined) do (lightListerIndex=findMenuItemIdx toolsMenu menuName_Tools_LightLister[i]))
			if (lightListerIndex != undefined) then (
				lightListerIndex += 1
			)
			else (
				lightListerIndex = -1
			)
			toolsMenu.addItem (menuMan.createActionItem "VRayLightLister" "VRay") lightListerIndex
			updateEnable = true
		)
		
		-- Add Vismat converter
		if ((findMenuItem toolsMenu "V-Ray vrmat converter") == undefined) do (
			toolsMenu.addItem (menuMan.createSeparatorItem()) -1	
			toolsMenu.addItem (menuMan.createActionItem "VRayVismatConverter" "VRay") -1
			updateEnable = true
		)
	)
	
	local createMenuItem
	for i=1 to menuName_Create.count do (if (createMenuItem==undefined) do (createMenuItem=findMenuItem mainMenuBar menuName_Create[i]))
	if (createMenuItem != undefined) do (
		local createMenu = createMenuItem.getSubMenu()
		
		-- Add geometry objects
		if ((findMenuItem createMenu "V-Ray") == undefined) do (
			subMenu = menuMan.createMenu "Create - V-Ray Primitives"
			-- Add items to the submenu
			appendMenuItem subMenu "VRayProxy" "VRay"
			appendMenuItem subMenu "VRayFur" "VRay"
			appendMenuItem subMenu "VRayMetaball" "VRay"
			appendMenuItem subMenu "VRaySphere" "VRay"
			appendMenuItem subMenu "VRayClipper" "VRay"
			appendMenuItem subMenu "VRayInstancer" "VRay"
			appendMenuItem subMenu "VRayPlane" "VRay"
			appendMenuItem subMenu "VRayVolumeGrid" "VRay"
			
			-- Add submenu before the first separator
			menuIndex = findFirstSeparatorIdx createMenu
			insertSubmenuItem createMenu "V-Ray" subMenu menuIndex
			updateEnable = true
		)
		
		-- Add light objects
		local lightsSubMenuItem
		for i=1 to menuName_Create_Lights.count do (if (lightsSubMenuItem==undefined) do (lightsSubMenuItem=findMenuItem createMenu menuName_Create_Lights[i]))
		if (lightsSubMenuItem != undefined) do (
			local lightsSubMenu = lightsSubMenuItem.getSubMenu()
			if ((findMenuItem lightsSubMenu "V-Ray") == undefined) do (
				subMenu = menuMan.createMenu "Create - Lights.V-Ray Lights"
				
				-- Add items to the submenu
				appendMenuItem subMenu "VRayLight_Plane" "VRay"
				appendMenuItem subMenu "VRayLight_Dome" "VRay"
				appendMenuItem subMenu "VRayLight_Sphere" "VRay"
				appendMenuItem subMenu "VRayLight_Mesh" "VRay"
				appendMenuItem subMenu "VRayIES" "VRay"
				appendMenuItem subMenu "VRayAmbientLight" "VRay"
				appendMenuItem subMenu "VRaySun" "VRay"
				
				-- Add submenu before the first separator
				menuIndex = findFirstSeparatorIdx lightsSubMenu
				insertSubmenuItem lightsSubMenu "V-Ray" subMenu menuIndex
				updateEnable = true
			)
		)
		
		
		-- Add camera objects
		local camerasSubMenuItem
		for i=1 to menuName_Create_Cameras.count do (if (camerasSubMenuItem==undefined) do (camerasSubMenuItem=findMenuItem createMenu menuName_Create_Cameras[i]))
		if (camerasSubMenuItem != undefined) do (
			local camerasSubMenu = camerasSubMenuItem.getSubMenu()
			if ((findMenuItem camerasSubMenu "V-Ray") == undefined) do (
				subMenu = menuMan.createMenu "Create - Cameras.V-Ray Cameras"
				
				-- Add items to the submenu
				appendMenuItem subMenu "VRayDomeCamera" "VRay"
				if (((maxVersion())[1]) < 18000) do (
					appendMenuItem subMenu "VRayPhysicalCamera" "VRay"
				)
				
				-- Add submenu before the first separator
				menuIndex = findFirstSeparatorIdx camerasSubMenu
				insertSubmenuItem camerasSubMenu "V-Ray" subMenu menuIndex
				updateEnable = true
			)
		)
		
		
		-- Add helper objects
		local helpersSubMenuItem
		for i=1 to menuName_Create_Helpers.count do (if (helpersSubMenuItem==undefined) do (helpersSubMenuItem=findMenuItem createMenu menuName_Create_Helpers[i]))
		if (helpersSubMenuItem != undefined) do (
			local helpersSubMenu = helpersSubMenuItem.getSubMenu()
			if ((findMenuItem helpersSubMenu "V-Ray") == undefined) do (
				subMenu = menuMan.createMenu "Create - Helpers.V-Ray Helpers"
				
				-- Add items to the submenu
				appendMenuItem subMenu "VRayLightMeter" "VRay"
				appendMenuItem subMenu "VRayBPTracer" "VRay"
				appendMenuItem subMenu "VRayStereoscopic" "VRay"
				
				-- Add submenu at the end
				helpersSubMenu.addItem (menuMan.createSeparatorItem()) -1
				insertSubmenuItem helpersSubMenu "V-Ray" subMenu -1
				updateEnable = true
			)
		)
		
		-- Add systems objects
		local systemsSubMenuItem
		for i=1 to menuName_Create_Systems.count do (if (systemsSubMenuItem==undefined) do (systemsSubMenuItem=findMenuItem createMenu menuName_Create_Systems[i]))
		if (systemsSubMenuItem != undefined) do (
			local systemsSubMenu = systemsSubMenuItem.getSubMenu()
			if ((findMenuItem systemsSubMenu "V-Ray") == undefined) do (
				subMenu = menuMan.createMenu "Create - Systems.V-Ray Systems"
				
				-- Add items to the submenu
				appendMenuItem subMenu "VRayStereoRig" "VRay"
				
				-- Add submenu at the end
				systemsSubMenu.addItem (menuMan.createSeparatorItem()) -1
				insertSubmenuItem systemsSubMenu "V-Ray" subMenu -1
				updateEnable = true
			)
		)
	)
	
	local modMenuItem
	for i=1 to menuName_Modifiers.count do (if (modMenuItem==undefined) do (modMenuItem=findMenuItem mainMenuBar menuName_Modifiers[i]))
	if (modMenuItem != undefined) do (
		local modMenu = modMenuItem.getSubMenu()
		
		-- Add modifiers
		if ((findMenuItem modMenu "V-Ray") == undefined) do (
			subMenu = menuMan.createMenu "Modifiers - V-Ray"
			-- Add items to the submenu
			appendMenuItem subMenu "VRayDisplacementMod" "VRay"
			appendMenuItem subMenu "VRayOrnatrixMod" "VRay"
			appendMenuItem subMenu "VRayHairFarmMod" "VRay"
			
			modMenu.addItem (menuMan.createSeparatorItem()) -1
			insertSubmenuItem modMenu "V-Ray" subMenu -1
			updateEnable = true
		)
	)
	
	local renderingMenuItem = undefined
	for i=1 to mainMenuBar_Rendering.count do (if (renderingMenuItem==undefined) do (renderingMenuItem=findMenuItem mainMenuBar mainMenuBar_Rendering[i]))
	if (renderingMenuItem != undefined) do (
		local renderingMenu = renderingMenuItem.getSubMenu()
		
		if ((findMenuItem renderingMenu "Make V-Ray RT Animation Preview") == undefined) do (
			-- Add item
			renderingMenu.addItem (menuMan.createSeparatorItem()) -1
			appendMenuItem renderingMenu "VRayRTAnimPreview" "VRay"
			
			updateEnable = true
		)
	)
	
	-- Update menu to show all newly added items
	if (updateEnable == true) do (
		menuMan.updateMenuBar()
	)
)

--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
-- ENHANCED MENUBAR
listLanguageIdxs = #("PTB", "ENU", "DEU", "FRA", "JPN", "KOR", "CHS")

-- Register V-Ray objects in enhanced menu bar
fn registerVRayEnhancedMenuBarItems = (

	local updateEnable = false
	local subMenu -- temporary submenu object
	local menuIndex -- temporary index used for item positioning calculation
	local subMenuItem
	local separatorIndex
	
	local mainMenuBar = menuMan.getMainMenuBar()
	
	--Get Language index in order to get corresponding to language, menu/item name
	local listLangInfo = sysinfo.GetMaxLanguage() -- #(9, 1, "ENU", "English (United States)", "en-US") - it is used the third element of language info list
	local languageIndex = findItem listLanguageIdxs listLangInfo[3]
	
	-- Add Light Lister item
	subMenuItem = findMenuItem mainMenuBar menuName_Enhanced_LightingCameras[languageIndex]
	if (subMenuItem != undefined) do (
		subMenu = subMenuItem.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Enhanced_LightingCameras_Tools[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local toolsMenu = subMenuItem.getSubMenu()
		
		if ((findMenuItem toolsMenu "V-Ray Light Lister") == undefined) do (
			menuIndex = findMenuItemIdx toolsMenu menuName_Tools_LightLister[languageIndex]
			if (menuIndex != undefined) then (
				menuIndex += 1
			)
			else (
				menuIndex = -1
			)
			
			toolsMenu.addItem (menuMan.createActionItem "VRayLightLister" "VRay")  menuIndex
			updateEnable = true
		)
	)
	
	-- Add Make V-Ray RT Animation Preview item
	subMenuItem = findMenuItem mainMenuBar mainMenuBar_Rendering[languageIndex]
	if (subMenuItem != undefined) do (
		subMenu = subMenuItem.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Enhanced_Rendering_Preview[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local renderingPreviewMenu = subMenuItem.getSubMenu()
		if ((findMenuItem renderingPreviewMenu "Make V-Ray RT Animation Preview") == undefined) do (
			
			-- Add item
			renderingPreviewMenu.addItem (menuMan.createSeparatorItem()) -1
			renderingPreviewMenu.addItem (menuMan.createActionItem "VRayRTAnimPreview" "VRay") -1
			
			updateEnable = true
			
		)
	)
	
	-- Add geometry objects vray menu
	local subMenuItemObjects =  findMenuItem mainMenuBar menuName_Enhanced_Objects[languageIndex]
	if (subMenuItemObjects != undefined) do (
		local mainObjMenu = subMenuItemObjects.getSubMenu()
		if ((findMenuItem mainObjMenu "V-Ray") == undefined) do (
			subMenu = menuMan.createMenu "Create - V-Ray Primitives"
			-- Add items to the submenu
			appendMenuItem subMenu "VRayProxy" "VRay"
			appendMenuItem subMenu "VRayFur" "VRay"
			appendMenuItem subMenu "VRayMetaball" "VRay"
			appendMenuItem subMenu "VRaySphere" "VRay"
			appendMenuItem subMenu "VRayClipper" "VRay"
			appendMenuItem subMenu "VRayInstancer" "VRay"
			appendMenuItem subMenu "VRayPlane" "VRay"
			appendMenuItem subMenu "VRayVolumeGrid" "VRay"
			
			-- Add submenu before the first separator
			menuIndex = findFirstSeparatorIdx mainObjMenu
			insertSubmenuItem mainObjMenu "V-Ray" subMenu menuIndex
			updateEnable = true
		)
	)
	
	-- Add light objects to  (Objects - Enhanced and Lighting/Cameras - Enhanced)Lights menu
	subMenuItem = undefined
	if (subMenuItemObjects != undefined) do (
		subMenu = subMenuItemObjects.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Create_Lights[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local lightsMenu = subMenuItem.getSubMenu()
		local lastSeparatorIdx = findLastSeparatorIdx lightsMenu
		
		if (lastSeparatorIdx <= 1) then lastSeparatorIdx = -1
		else lastSeparatorIdx -= 1
		
		lightsMenu.addItem (menuMan.createSeparatorItem()) lastSeparatorIdx
		lastSeparatorIdx += 1
		if ((findMenuItem lightsMenu "V-Ray Plane Light") == undefined) do (		
			lightsMenu.addItem (menuMan.createActionItem "VRayLight_Plane" "VRay") lastSeparatorIdx
			updateEnable = true
		)
		lastSeparatorIdx += 1
		if ((findMenuItem lightsMenu "V-Ray Dome Light") == undefined) do (		
			lightsMenu.addItem (menuMan.createActionItem "VRayLight_Dome" "VRay") lastSeparatorIdx
			updateEnable = true
		)
		lastSeparatorIdx += 1
		if ((findMenuItem lightsMenu "V-Ray Sphere Light") == undefined) do (
			lightsMenu.addItem (menuMan.createActionItem "VRayLight_Sphere" "VRay") lastSeparatorIdx
			updateEnable = true
		)
		lastSeparatorIdx += 1
		if ((findMenuItem lightsMenu "V-Ray Mesh Light") == undefined) do (	
			lightsMenu.addItem (menuMan.createActionItem "VRayLight_Mesh" "VRay") lastSeparatorIdx
			updateEnable = true
		)
		lastSeparatorIdx += 1
		if ((findMenuItem lightsMenu "V-Ray IES Light") == undefined) do (
			lightsMenu.addItem (menuMan.createActionItem "VRayIES" "VRay") lastSeparatorIdx
			updateEnable = true
		)
		lastSeparatorIdx += 1
		if ((findMenuItem lightsMenu "V-Ray Ambient Light") == undefined) do (
			lightsMenu.addItem (menuMan.createActionItem "VRayAmbientLight" "VRay") lastSeparatorIdx
			updateEnable = true
		)
		lastSeparatorIdx += 1
		if ((findMenuItem lightsMenu "V-Ray Sun") == undefined) do (
			lightsMenu.addItem (menuMan.createActionItem "VRaySun" "VRay") lastSeparatorIdx
			updateEnable = true
		)
	)
	
	-- Add camera objects to (Objects - Enhanced and Lighting/Cameras - Enhanced)/Cameras menu
	subMenuItem = undefined
	if (subMenuItemObjects != undefined) do (
		subMenu = subMenuItemObjects.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Create_Cameras[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local camerasMenu = subMenuItem.getSubMenu()
		separatorIndex = findFirstSeparatorIdx camerasMenu
		camerasMenu.addItem (menuMan.createSeparatorItem()) separatorIndex
		if ((findMenuItem camerasMenu "V-Ray Dome Camera") == undefined) do (
			separatorIndex += 1
			camerasMenu.addItem (menuMan.createActionItem "VRayDomeCamera" "VRay") separatorIndex
			updateEnable = true
		)
		if (((maxVersion())[1]) < 18000) do (
			if ((findMenuItem camerasMenu "V-Ray Physical Camera") == undefined) do (
				separatorIndex += 1
				camerasMenu.addItem (menuMan.createActionItem "VRayPhysicalCamera" "VRay") separatorIndex
				updateEnable = true
			)
		)
	)
	
		
	-- Add helper objects vray menu
	subMenuItem = undefined
	if (subMenuItemObjects != undefined) do (
		subMenu = subMenuItemObjects.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Create_Helpers[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local helpersMenu = subMenuItem.getSubMenu()
		if ((findMenuItem helpersMenu "V-Ray Light Meter") == undefined) do (
			helpersMenu.addItem (menuMan.createSeparatorItem()) -1
			helpersMenu.addItem (menuMan.createActionItem "VRayLightMeter" "VRay") -1
			updateEnable = true
		)
		if ((findMenuItem helpersMenu "V-Ray BPTracer") == undefined) do (
			helpersMenu.addItem (menuMan.createSeparatorItem()) -1
			helpersMenu.addItem (menuMan.createActionItem "VRayBPTracer" "VRay") -1
			updateEnable = true
		)
		if ((findMenuItem helpersMenu "V-Ray Stereoscopic") == undefined) do (
			helpersMenu.addItem (menuMan.createSeparatorItem()) -1
			helpersMenu.addItem (menuMan.createActionItem "VRayStereoscopic" "VRay") -1
			updateEnable = true
		)
	)
	
	--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	
	-- Add modifiers vray menu
	subMenuItem = findMenuItem mainMenuBar menuName_Modifiers[languageIndex]
	if (subMenuItem != undefined) do (
		local modMenu = subMenuItem.getSubMenu()
		if ((findMenuItem modMenu "V-Ray") == undefined) do (
			subMenu = menuMan.createMenu "Modifiers - V-Ray"
			-- Add items to the submenu
			appendMenuItem subMenu "VRayDisplacementMod" "VRay"
			appendMenuItem subMenu "VRayOrnatrixMod" "VRay"
			
			modMenu.addItem (menuMan.createSeparatorItem()) -1
			insertSubmenuItem modMenu "V-Ray" subMenu -1
			updateEnable = true
		)
	)
	
	--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	
	-- Add vray vismat convertor
	subMenuItem = findMenuItem mainMenuBar menuName_Enhanced_Materials[languageIndex]
	if (subMenuItem != undefined) do (
		subMenu = subMenuItem.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Enhanced_Materials_Tools[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local toolsMaterialMenu = subMenuItem.getSubMenu()
		if ((findMenuItem toolsMaterialMenu "V-Ray vrmat converter") == undefined) do (
			
			-- Add item
			toolsMaterialMenu.addItem (menuMan.createSeparatorItem()) -1
			toolsMaterialMenu.addItem (menuMan.createActionItem "VRayVismatConverter" "VRay") -1
			
			updateEnable = true
			
		)
	)
	
	-- Update menu to show all newly added items
	if (updateEnable == true) do (
		menuMan.updateMenuBar()
	)
)

fn registerVRayQuadMenuItems = (
	local quadMenu=menuMan.getViewportRightClickMenu #nonePressed
	if (quadMenu!=undefined) do (
		local menu=quadMenu.getMenu 1
		if (menu!=undefined) do (
			-- add VRayProperties
			local menuIndex=findMenuItemIdx menu "V-Ray properties"
			if (menuIndex == undefined) do (
				appendMenuItem menu "VRayProperties" "VRay"
			)
			-- add VRayShowVFB
			menuIndex=findMenuItemIdx menu "V-Ray VFB"
			if (menuIndex == undefined) do (
				local menuVRPropertiesIdx=findMenuItemIdx menu "V-Ray properties"
				if (menuVRPropertiesIdx != undefined) then (
					local vrayVFB=menuMan.createActionItem "VRayShowVFB" "VRay"
					menu.addItem vrayVFB (menuVRPropertiesIdx + 1)
				) else (
					appendMenuItem menu "VRayShowVFB" "VRay"
				)
			)
			-- add VRaySceneConverter
			menuIndex=findMenuItemIdx menu "V-Ray scene converter"
			if (menuIndex == undefined) do (
				local menuVRVFBIdx=findMenuItemIdx menu "V-Ray VFB"
				if (menuVRVFBIdx != undefined) then (
					local vraySceneConvert=menuMan.createActionItem "VRaySceneConverter" "VRay"
					menu.addItem vraySceneConvert (menuVRVFBIdx + 1)
				) else (
					appendMenuItem menu "VRaySceneConverter" "VRay"
				)
			)
			-- add Bitmap2VRayHDRI texture converter
			menuIndex=findMenuItemIdx menu "V-Ray Bitmap to VRayHDRI converter"
			if (menuIndex == undefined) do (
				local menuVRSceneConverterIdx=findMenuItemIdx menu "V-Ray scene converter"
				if (menuVRSceneConverterIdx != undefined) then (
					local vrBitmap2VRayHDRI=menuMan.createActionItem "VRayBitmap2VRayHDRITexmapConverter" "VRay"
					menu.addItem vrBitmap2VRayHDRI (menuVRSceneConverterIdx + 1)
				) else (
					appendMenuItem menu "VRayBitmap2VRayHDRITexmapConverter" "VRay"
				)
			)
			-- add VRayMeshExport
			menuIndex=findMenuItemIdx menu "V-Ray mesh export"
			if (menuIndex == undefined) do (
				local menuVRBmp2HDRIConverterIdx=findMenuItemIdx menu "V-Ray Bitmap to VRayHDRI converter"
				if (menuVRBmp2HDRIConverterIdx != undefined) then (
					local vrayMeshExp=menuMan.createActionItem "VRayMeshExport" "VRay"
					menu.addItem vrayMeshExp (menuVRBmp2HDRIConverterIdx + 1)
				) else (
					appendMenuItem menu "VRayMeshExport" "VRay"
				)
			)
			-- add vrsceneExport
			menuIndex=findMenuItemIdx menu ".vrscene exporter"
			if (menuIndex == undefined) do (
				local menuVRMeshExpIdx=findMenuItemIdx menu "V-Ray mesh export"
				if (menuVRMeshExpIdx != undefined) then (
					local vrayVRSceneExp=menuMan.createActionItem "vrsceneExport" "VRay"
					menu.addItem vrayVRSceneExp (menuVRMeshExpIdx + 1)
				) else (
					appendMenuItem menu "vrsceneExport" "VRay"
				)
			)
		)
	)
)

-- Unregister V-Ray objects from menu bar
fn unregisterVRayMenuBarItems = (
	local mainMenuBar = menuMan.getMainMenuBar()
	local vrayMenuItemIdx
	
	local updateEnable = false
	
	local toolsMenuItem
	for i=1 to menuName_Tools.count do (if (toolsMenuItem==undefined) do (toolsMenuItem=findMenuItem mainMenuBar menuName_Tools[i]))
	if (toolsMenuItem != undefined) do (
		local toolsMenu = toolsMenuItem.getSubMenu()
		if (toolsMenu != undefined) do (
			vrayMenuItemIdx = findMenuItemIdx toolsMenu "V-Ray Light Lister"
			if (vrayMenuItemIdx != undefined) do (
				toolsMenu.removeItemByPosition (vrayMenuItemIdx)
				updateEnable = true
			)
			
			vrayMenuItemIdx = findMenuItemIdx toolsMenu "V-Ray vrmat converter"
			if (vrayMenuItemIdx != undefined) do (
				toolsMenu.removeItemByPosition (vrayMenuItemIdx)
				if (vrayMenuItemIdx > 1) do (
					if ((toolsMenu.getItem (vrayMenuItemIdx-1)).getIsSeparator()) do (
						toolsMenu.removeItemByPosition (vrayMenuItemIdx-1)
						updateEnable = true
					)
				)
			)
		)
	)
	
	local createMenuItem
	for i=1 to menuName_Create.count do (if (createMenuItem==undefined) do (createMenuItem=findMenuItem mainMenuBar menuName_Create[i]))
	if(createMenuItem != undefined) do (
		local createMenu = createMenuItem.getSubMenu()
		local helpersMenuItem
		for i=1 to menuName_Create_Helpers.count do (if (helpersMenuItem==undefined) do (helpersMenuItem=findMenuItem createMenu menuName_Create_Helpers[i]))
		if(helpersMenuItem != undefined) do (
			local helpersMenu = helpersMenuItem.getSubMenu()
			vrayMenuItemIdx = findMenuItemIdx helpersMenu "V-Ray"
			if ((vrayMenuItemIdx != undefined) and (vrayMenuItemIdx > 1)) do (
				if((helpersMenu.getItem (vrayMenuItemIdx-1)).getIsSeparator()) do (
					helpersMenu.removeItemByPosition (vrayMenuItemIdx-1)
					updateEnable = true
				)
			)
		)
	)
	
	local modMenuItem
	for i=1 to menuName_Modifiers.count do (if (modMenuItem==undefined) do (modMenuItem=findMenuItem mainMenuBar menuName_Modifiers[i]))
	if (modMenuItem != undefined) do (
		local modMenu = modMenuItem.getSubMenu()
		vrayMenuItemIdx = findMenuItemIdx modMenu "V-Ray"
		if ((vrayMenuItemIdx != undefined) and (vrayMenuItemIdx > 1)) do (
			if ((modMenu.getItem (vrayMenuItemIdx-1)).getIsSeparator()) do (
				modMenu.removeItemByPosition (vrayMenuItemIdx-1)
				updateEnable = true
			)
		)
	)
	
	local renderingMenuItem = undefined
	for i=1 to mainMenuBar_Rendering.count do (if (renderingMenuItem==undefined) do (renderingMenuItem=findMenuItem mainMenuBar mainMenuBar_Rendering[i]))
	if (renderingMenuItem != undefined) do (
		local renderingMenu = renderingMenuItem.getSubMenu()
		vrayMenuItemIdx = findMenuItemIdx renderingMenu "Make V-Ray RT Animation Preview"
		if (vrayMenuItemIdx != undefined) do (
			renderingMenu.removeItemByPosition vrayMenuItemIdx
			if (vrayMenuItemIdx > 1) do (
				if ((renderingMenu.getItem (vrayMenuItemIdx-1)).getIsSeparator()) do (
					renderingMenu.removeItemByPosition (vrayMenuItemIdx-1)
				)
			)
			updateEnable = true
		)
	)
	
	-- unregister all V-Ray submenus
	local vrayMenu
	vrayMenu = menuMan.findMenu "Create - V-Ray Primitives"
	if (vrayMenu != undefined) do (
		menuMan.unregisterMenu vrayMenu
		updateEnable = true
	)

	vrayMenu = menuMan.findMenu "Create - Lights.V-Ray Lights"
	if (vrayMenu != undefined) do (
		menuMan.unregisterMenu vrayMenu
		updateEnable = true
	)

	vrayMenu = menuMan.findMenu "Create - Cameras.V-Ray Cameras"
	if (vrayMenu != undefined) do (
		menuMan.unregisterMenu vrayMenu
		updateEnable = true
	)

	vrayMenu = menuMan.findMenu "Create - Helpers.V-Ray Helpers"
	if (vrayMenu != undefined) do (
		menuMan.unregisterMenu vrayMenu
		updateEnable = true
	)

	vrayMenu = menuMan.findMenu "Modifiers - V-Ray"
	if (vrayMenu != undefined) do (
		menuMan.unregisterMenu vrayMenu
		updateEnable = true
	)


	-- Update menu to show all newly added items
	if (updateEnable == true) do (
		menuMan.updateMenuBar()
	)
)

fn unregisterVRayQuadMenuItems = (
	local quadMenu=menuMan.getViewportRightClickMenu #nonePressed
	if (quadMenu!=undefined) do (
		local menu=quadMenu.getMenu 1
		if (menu!=undefined) do (
			-- remove VRayProperties
			local menuIndex=findMenuItemIdx menu "V-Ray properties"
			if (menuIndex!=undefined) do (
				menu.removeItemByPosition menuIndex
			)
			-- remove VRayShowVFB
			menuIndex=findMenuItemIdx menu "V-Ray VFB"
			if (menuIndex!=undefined) do (
				menu.removeItemByPosition menuIndex
			)
			-- remove VRaySceneConverter
			menuIndex=findMenuItemIdx menu "V-Ray scene converter"
			if (menuIndex!=undefined) do (
				menu.removeItemByPosition menuIndex
			)
			-- remove Bitmap2VRayHDRI texture converter
			menuIndex=findMenuItemIdx menu "V-Ray Bitmap to VRayHDRI converter"
			if (menuIndex!=undefined) do (
				menu.removeItemByPosition menuIndex
			)
			-- remove VRayMeshExport
			menuIndex=findMenuItemIdx menu "V-Ray mesh export"
			if (menuIndex!=undefined) do (
				menu.removeItemByPosition menuIndex
			)
			-- remove vrsceneExport
			menuIndex=findMenuItemIdx menu ".vrscene exporter"
			if (menuIndex!=undefined) do (
				menu.removeItemByPosition menuIndex
			)
		)
	)
)

fn unregisterVRayEnhancedMenuBarItems = (
	local updateEnable = false
	local subMenu -- temporary submenu object
	local menuIndex -- temporary index used for item positioning calculation
	local subMenuItem
	local separatorIndex
	
	local mainMenuBar = menuMan.getMainMenuBar()
	
	--Get Language index in order to get corresponding to language, menu/item name
	local listLangInfo = sysinfo.GetMaxLanguage() -- #(9, 1, "ENU", "English (United States)", "en-US") - it is used the third element of language info list
	local languageIndex = findItem listLanguageIdxs listLangInfo[3]
	
	-- Remove Light Lister item
	subMenuItem = findMenuItem mainMenuBar menuName_Enhanced_LightingCameras[languageIndex]
	if (subMenuItem != undefined) do (
		subMenu = subMenuItem.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Enhanced_LightingCameras_Tools[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local toolsMenu = subMenuItem.getSubMenu()
		menuIndex = findMenuItemIdx toolsMenu "V-Ray Light Lister"
		if (menuIndex != undefined) do (
			toolsMenu.removeItemByPosition (menuIndex)
			updateEnable = true
		)
	)
	
	-- Remove Make V-Ray RT Animation Preview item
	subMenuItem = findMenuItem mainMenuBar mainMenuBar_Rendering[languageIndex]
	if (subMenuItem != undefined) do (
		subMenu = subMenuItem.getSubMenu()
		--menuIndex = findMenuItemIdx subMenu "Make V-Ray RT Animation Preview"
		--if (menuIndex != undefined) do (
		--	subMenu.removeItemByPosition (menuIndex)
		--	updateEnable = true
		--)
		subMenuItem =  findMenuItem subMenu menuName_Enhanced_Rendering_Preview[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local renderingPreviewMenu = subMenuItem.getSubMenu()
		menuIndex = findMenuItemIdx renderingPreviewMenu "Make V-Ray RT Animation Preview"
		if (menuIndex != undefined) do (
			renderingPreviewMenu.removeItemByPosition(menuIndex)
			if ((renderingPreviewMenu.getItem (menuIndex-1)).getIsSeparator()) do (
				renderingPreviewMenu.removeItemByPosition (menuIndex-1)
			)
			
			updateEnable = true
			
		)
	)
	
	-- Remove geometry objects vray menu
	local subMenuItemObjects =  findMenuItem mainMenuBar menuName_Enhanced_Objects[languageIndex]
	if (subMenuItemObjects != undefined) do (
		local mainObjMenu = subMenuItemObjects.getSubMenu()
		subMenuItem = findMenuItem mainObjMenu "V-Ray"
		if (subMenuItem != undefined) do (
			subMenu = subMenuItem.getSubMenu()
			
			menuIndex = findMenuItemIdx subMenu "V-Ray Proxy"
			if (menuIndex != undefined) do (
				subMenu.removeItemByPosition(menuIndex)
			)
			
			menuIndex = findMenuItemIdx subMenu "V-Ray Fur"
			if (menuIndex != undefined) do (
				subMenu.removeItemByPosition(menuIndex)
			)
			
			menuIndex = findMenuItemIdx subMenu "V-Ray Plane"
			if (menuIndex != undefined) do (
				subMenu.removeItemByPosition(menuIndex)
			)
			
			menuIndex = findMenuItemIdx subMenu "V-Ray Sphere"
			if (menuIndex != undefined) do (
				subMenu.removeItemByPosition(menuIndex)
			)
			
			menuMan.unregisterMenu subMenu
			updateEnable = true
		)
	)
	
	-- Remove light objects from  (Objects - Enhanced and Lighting/Cameras - Enhanced)Lights menu
	subMenuItem = undefined
	if (subMenuItemObjects != undefined) do (
		subMenu = subMenuItemObjects.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Create_Lights[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local lightsMenu = subMenuItem.getSubMenu()
		menuIndex = findMenuItemIdx lightsMenu "V-Ray Plane Light"
		if (menuIndex != undefined) do (
			lightsMenu.removeItemByPosition(menuIndex)
			if ((lightsMenu.getItem (menuIndex-1)).getIsSeparator()) do (
				lightsMenu.removeItemByPosition(menuIndex-1)
			)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx lightsMenu "V-Ray Dome Light"
		if (menuIndex != undefined) do (		
			lightsMenu.removeItemByPosition(menuIndex)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx lightsMenu "V-Ray Sphere Light"
		if (menuIndex != undefined) do (
			lightsMenu.removeItemByPosition(menuIndex)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx lightsMenu "V-Ray Mesh Light"
		if (menuIndex != undefined) do (	
			lightsMenu.removeItemByPosition(menuIndex)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx lightsMenu "V-Ray IES Light"
		if (menuIndex != undefined) do (
			lightsMenu.removeItemByPosition(menuIndex)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx lightsMenu "V-Ray Ambient Light"
		if (menuIndex != undefined) do (
			lightsMenu.removeItemByPosition(menuIndex)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx lightsMenu "V-Ray Sun"
		if (menuIndex != undefined) do (
			lightsMenu.removeItemByPosition(menuIndex)
			updateEnable = true
		)
	)
	
	-- Remove camera objects from (Objects - Enhanced and Lighting/Cameras - Enhanced)/Cameras menu
	subMenuItem = undefined
	if (subMenuItemObjects != undefined) do (
		subMenu = subMenuItemObjects.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Create_Cameras[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local camerasMenu = subMenuItem.getSubMenu()
		menuIndex = findMenuItemIdx camerasMenu "V-Ray Dome Camera"
		if (menuIndex != undefined) do (
			camerasMenu.removeItemByPosition(menuIndex)
			if ((camerasMenu.getItem (menuIndex-1)).getIsSeparator()) do (
				camerasMenu.removeItemByPosition(menuIndex-1)
			)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx camerasMenu "V-Ray Physical Camera"
		if (menuIndex != undefined) do (
			camerasMenu.removeItemByPosition(menuIndex)
			updateEnable = true
		)
	)
	
		
	-- Remove helper objects vray menu
	subMenuItem = undefined
	if (subMenuItemObjects != undefined) do (
		subMenu = subMenuItemObjects.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Create_Helpers[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local helpersMenu = subMenuItem.getSubMenu()
		menuIndex = findMenuItemIdx helpersMenu "V-Ray Light Meter"
		if (menuIndex != undefined) do (
			helpersMenu.removeItemByPosition(menuIndex)
			if ((helpersMenu.getItem (menuIndex-1)).getIsSeparator()) do (
				helpersMenu.removeItemByPosition(menuIndex-1)
			)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx helpersMenu "V-Ray BPTracer"
		if (menuIndex != undefined) do (
			helpersMenu.removeItemByPosition(menuIndex)
			if ((helpersMenu.getItem (menuIndex-1)).getIsSeparator()) do (
				helpersMenu.removeItemByPosition(menuIndex-1)
			)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx helpersMenu "V-Ray Stereoscopic"
		if (menuIndex != undefined) do (
			helpersMenu.removeItemByPosition(menuIndex)
			if ((helpersMenu.getItem (menuIndex-1)).getIsSeparator()) do (
				helpersMenu.removeItemByPosition(menuIndex-1)
			)
			updateEnable = true
		)
	)
	
	--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	
	-- Remove modifiers vray menu
	subMenuItem = findMenuItem mainMenuBar menuName_Modifiers[languageIndex]
	if (subMenuItem != undefined) do (
		local modMenu = subMenuItem.getSubMenu()
		local subMenuItemIdx = findMenuItemIdx modMenu "V-Ray"
		
		if ((subMenuItemIdx != undefined) and  (modMenu.getItem (subMenuItemIdx-1)).getIsSeparator()) do (
			modMenu.removeItemByPosition(subMenuItemIdx-1)
		)
		
		subMenuItem = findMenuItem modMenu "V-Ray"
		if (subMenuItem != undefined) do (
			subMenu = subMenuItem.getSubMenu()
			
			menuIndex = findMenuItemIdx subMenu "V-Ray Displacement Mod"
			if (menuIndex != undefined) do (
				subMenu.removeItemByPosition(menuIndex)
			)
			menuIndex = findMenuItemIdx subMenu "V-Ray Ornatrix Mod"
			if (menuIndex != undefined) do (
				subMenu.removeItemByPosition(menuIndex)
			)
			
			menuMan.unregisterMenu subMenu
			updateEnable = true
		)
	)
	
	--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	
	-- Remove vray vismat convertor
	subMenuItem = findMenuItem mainMenuBar menuName_Enhanced_Materials[languageIndex]
	if (subMenuItem != undefined) do (
		subMenu = subMenuItem.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Enhanced_Materials_Tools[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local toolsMaterialMenu = subMenuItem.getSubMenu()
		menuIndex = findMenuItemIdx toolsMaterialMenu "V-Ray vrmat converter"
		if (menuIndex != undefined) do (
			toolsMaterialMenu.removeItemByPosition (menuIndex)
			updateEnable = true
		)
	)
	
	if (updateEnable == true) do (
		menuMan.updateMenuBar()
	)
)

fn registerVRayMenus = (
	local currentWorkspace = WorkspaceManager.GetCurrentWorkspaceIdentifier()
	if (currentWorkspace == "Workspace1") do (
		registerVRayMenuBarItems()
	)
	if (currentWorkspace == "Workspace1Enhanced") do (
		registerVRayEnhancedMenuBarItems()
	)
	registerVRayQuadMenuItems()
	
	menuMan.saveMenuFile (menuMan.getMenuFile())
)

fn unregisterVRayMenus = (
	local currentWorkspace = WorkspaceManager.GetCurrentWorkspaceIdentifier()
	if (currentWorkspace == "Workspace1") do (
		unregisterVRayMenuBarItems()
	)
	if (currentWorkspace == "Workspace1Enhanced") do (
		unregisterVRayEnhancedMenuBarItems()
	)
	unregisterVRayQuadMenuItems()
)

fn updateVRayMenus = (
	unregisterVRayMenus()
	registerVRayMenus()
)

